@@ -46,7 +46,7 @@ typedef enum
 // Fallback IWADs to use if no IWADs are detected.
 
 static const iwad_t fallback_iwads[] = {
-    { "doom.wad", doom, retail, vanilla, "Doom" }
+    { "doom.wad", doom, retail, "Doom" }
 };
 
 // Array of IWADs found to be installed
@@ -136,7 +136,7 @@ static void AddWADs(execute_context_t *exec)
                 have_wads = 1;
             }
 
-            AddCmdLineParameter(exec, "%s", wads[i]);
+            AddCmdLineParameter(exec, "\"%s\"", wads[i]);
         }
     }
 }
