@@ -27,11 +27,12 @@
 #include "doomdata.h"
 #include "doomdef.h"
 #include "doomstat.h"
-#include "hu_stuff.h"
+#include "doomtype.h"
 #include "i_video.h"
+#include "m_config.h"
 #include "m_input.h"
-#include "m_menu.h"
-#include "m_misc2.h"
+#include "mn_menu.h"
+#include "m_misc.h"
 #include "p_maputl.h"
 #include "p_mobj.h"
 #include "p_setup.h"
@@ -41,45 +42,53 @@
 #include "r_state.h"
 #include "r_things.h"
 #include "st_stuff.h"
+#include "st_widgets.h"
 #include "tables.h"
 #include "v_flextran.h"
+#include "v_fmt.h"
 #include "v_video.h"
-#include "w_wad.h"
+#include "ws_stuff.h"
 #include "z_zone.h"
 
 //jff 1/7/98 default automap colors added
-int mapcolor_back;    // map background
-int mapcolor_grid;    // grid lines color
-int mapcolor_wall;    // normal 1s wall color
-int mapcolor_fchg;    // line at floor height change color
-int mapcolor_cchg;    // line at ceiling height change color
-int mapcolor_clsd;    // line at sector with floor=ceiling color
-int mapcolor_rkey;    // red key color
-int mapcolor_bkey;    // blue key color
-int mapcolor_ykey;    // yellow key color
-int mapcolor_rdor;    // red door color  (diff from keys to allow option)
-int mapcolor_bdor;    // blue door color (of enabling one but not other )
-int mapcolor_ydor;    // yellow door color
-int mapcolor_tele;    // teleporter line color
-int mapcolor_secr;    // secret sector boundary color
-int mapcolor_revsecr; // revealed secret sector boundary color
-int mapcolor_exit;    // jff 4/23/98 add exit line color
-int mapcolor_unsn;    // computer map unseen line color
-int mapcolor_flat;    // line with no floor/ceiling changes
-int mapcolor_sprt;    // general sprite color
-int mapcolor_hair;    // crosshair color
-int mapcolor_sngl;    // single player arrow color
-int mapcolor_plyr[4]; // colors for player arrows in multiplayer
-int mapcolor_frnd;    // colors for friends of player
-int mapcolor_item;    // item sprite color
-int mapcolor_enemy;   // enemy sprite color
+static int mapcolor_back;    // map background
+static int mapcolor_grid;    // grid lines color
+static int mapcolor_wall;    // normal 1s wall color
+static int mapcolor_fchg;    // line at floor height change color
+static int mapcolor_cchg;    // line at ceiling height change color
+static int mapcolor_clsd;    // line at sector with floor=ceiling color
+static int mapcolor_rkey;    // red key color
+static int mapcolor_bkey;    // blue key color
+static int mapcolor_ykey;    // yellow key color
+static int mapcolor_rdor;    // red door color  (diff from keys to allow option)
+static int mapcolor_bdor;    // blue door color (of enabling one but not other )
+static int mapcolor_ydor;    // yellow door color
+static int mapcolor_tele;    // teleporter line color
+static int mapcolor_secr;    // secret sector boundary color
+static int mapcolor_revsecr; // revealed secret sector boundary color
+static int mapcolor_exit;    // jff 4/23/98 add exit line color
+static int mapcolor_unsn;    // computer map unseen line color
+static int mapcolor_flat;    // line with no floor/ceiling changes
+static int mapcolor_sprt;    // general sprite color
+static int mapcolor_hair;    // crosshair color
+static int mapcolor_sngl;    // single player arrow color
+static int mapcolor_plyr[4]; // colors for player arrows in multiplayer
+static int mapcolor_frnd;    // colors for friends of player
+static int mapcolor_item;    // item sprite color
+static int mapcolor_enemy;   // enemy sprite color
 
 //jff 3/9/98 add option to not show secret sectors until entered
-int map_secret_after;
+static boolean map_secret_after;
 
-int map_keyed_door; // keyed doors are colored or flashing
+enum {
+  MAP_KEYED_DOOR_OFF,
+  MAP_KEYED_DOOR_COLOR,
+  MAP_KEYED_DOOR_FLASH
+};
 
-int map_smooth_lines;
+static int map_keyed_door; // keyed doors are colored or flashing
+
+static boolean map_smooth_lines;
 
 // [Woof!] FRACTOMAPBITS: overflow-safe coordinate system.
 // Written by Andrey Budko (entryway), adapted from prboom-plus/src/am_map.*
@@ -209,7 +218,7 @@ static mline_t thintriangle_guy[] =
 
 int ddt_cheating = 0;         // killough 2/7/98: make global, rename to ddt_*
 
-int automap_grid = 0;
+boolean automap_grid = false;
 
 boolean automapactive = false;
 static boolean automapfirststart = true;
@@ -248,9 +257,7 @@ static fixed_t  max_w;          // max_x-min_x,
 static fixed_t  max_h;          // max_y-min_y
 
 // based on player size
-static fixed_t  min_w;
-static fixed_t  min_h;
-
+static const fixed_t min_h = 2 * MAPPLAYERRADIUS; // const? never changed?
 
 static fixed_t  min_scale_mtof; // used to tell when to stop zooming out
 static fixed_t  max_scale_mtof; // used to tell when to stop zooming in
@@ -274,7 +281,7 @@ static patch_t *marknums[10];   // numbers used for marking by the automap
 mpoint_t *markpoints = NULL;    // where the points are
 int markpointnum = 0; // next point to be assigned (also number of points now)
 int markpointnum_max = 0;       // killough 2/22/98
-int followplayer = 1; // specifies whether to follow the player around
+boolean followplayer = true; // specifies whether to follow the player around
 
 static boolean stopped = true;
 
@@ -283,16 +290,27 @@ static void AM_drawFline_Vanilla(fline_t* fl, int color);
 static void AM_drawFline_Smooth(fline_t* fl, int color);
 void (*AM_drawFline)(fline_t*, int) = AM_drawFline_Vanilla;
 
-// [crispy] automap rotate mode needs these early on
-boolean automaprotate = false;
+// [crispy/Woof!] automap rotate mode and square aspect ratio need these early on
+static boolean automaprotate = false;
+static boolean automapsquareaspect = false;
+#define ADJUST_ASPECT_RATIO (correct_aspect_ratio && automapsquareaspect)
 static void AM_rotate(int64_t *x, int64_t *y, angle_t a);
-static void AM_rotatePoint(mpoint_t *pt);
+static void AM_transformPoint(mpoint_t *pt);
 static mpoint_t mapcenter;
 static angle_t mapangle;
 
-// [FG] prev/next weapon keys and buttons
-extern int mousebprevweapon;
-extern int mousebnextweapon;
+enum
+{
+  PAN_UP,
+  PAN_DOWN,
+  PAN_LEFT,
+  PAN_RIGHT,
+  ZOOM_IN,
+  ZOOM_OUT,
+  STATE_NUM
+};
+
+static int buttons_state[STATE_NUM] = { 0 };
 
 //
 // AM_activateNewScale()
@@ -391,11 +409,25 @@ static void AM_addMark(void)
 //
 // Passed nothing, returns nothing
 //
+
+static void AM_setMinMaxScale(void)
+{
+  const fixed_t a = FixedDiv(f_w << FRACBITS, max_w);
+  const fixed_t b = FixedDiv(f_h << FRACBITS, max_h);
+
+  min_scale_mtof = (a < b) ? a : b;
+  max_scale_mtof = FixedDiv(f_h << FRACBITS, min_h);
+
+  if (scale_mtof > max_scale_mtof)
+    scale_mtof = max_scale_mtof;
+
+  if (scale_mtof < min_scale_mtof)
+    scale_mtof = min_scale_mtof;
+}
+
 static void AM_findMinMaxBoundaries(void)
 {
   int i;
-  fixed_t a;
-  fixed_t b;
 
   min_x = min_y =  INT_MAX;
   max_x = max_y = -INT_MAX;
@@ -417,14 +449,7 @@ static void AM_findMinMaxBoundaries(void)
   max_w = (max_x >>= FRACTOMAPBITS) - (min_x >>= FRACTOMAPBITS);
   max_h = (max_y >>= FRACTOMAPBITS) - (min_y >>= FRACTOMAPBITS);
 
-  min_w = 2*MAPPLAYERRADIUS; // const? never changed?
-  min_h = 2*MAPPLAYERRADIUS;
-
-  a = FixedDiv(f_w<<FRACBITS, max_w);
-  b = FixedDiv(f_h<<FRACBITS, max_h);
-
-  min_scale_mtof = a < b ? a : b;
-  max_scale_mtof = FixedDiv(f_h<<FRACBITS, 2*MAPPLAYERRADIUS);
+  AM_setMinMaxScale();
 }
 
 void AM_SetMapCenter(fixed_t x, fixed_t y)
@@ -495,7 +520,7 @@ static void AM_changeWindowLoc(void)
 //
 void AM_initVariables(void)
 {
-  static event_t st_notify = { ev_keyup, AM_MSGENTERED };
+  static event_t st_notify = {.type = ev_keyup, .data1.i = AM_MSGENTERED};
 
   automapactive = true;
 
@@ -544,7 +569,7 @@ static void AM_loadPics(void)
   for (i=0;i<10;i++)
   {
     M_snprintf(namebuf, sizeof(namebuf), "AMMNUM%d", i);
-    marknums[i] = W_CacheLumpName(namebuf, PU_STATIC);
+    marknums[i] = V_CachePatchName(namebuf, PU_STATIC);
   }
 }
 
@@ -583,7 +608,7 @@ static void AM_clearLastMark(void)
     markpointnum--;
 }
 
-void AM_EnableSmoothLines(void)
+static void AM_EnableSmoothLines(void)
 {
   AM_drawFline = map_smooth_lines ? AM_drawFline_Smooth : AM_drawFline_Vanilla;
 }
@@ -599,7 +624,7 @@ static void AM_initScreenSize(void)
   if (automapoverlay && scaledviewheight == SCREENHEIGHT)
     f_h = video.height;
   else
-    f_h = video.height - V_ScaleY(ST_HEIGHT);
+    f_h = V_ScaleY(SCREENHEIGHT - ST_HEIGHT);
 }
 
 void AM_ResetScreenSize(void)
@@ -612,6 +637,9 @@ void AM_ResetScreenSize(void)
   {
     // Change the scaling multipliers
     scale_mtof = FixedDiv(f_w << FRACBITS, m_w);
+
+    AM_setMinMaxScale();
+
     scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
   }
 
@@ -665,7 +693,9 @@ static void AM_LevelInit(void)
 //
 void AM_Stop (void)
 {
-  static event_t st_notify = { 0, ev_keyup, AM_MSGEXITED };
+  static event_t st_notify = {.type = 0, .data1.i = ev_keyup, .data2.i = AM_MSGEXITED};
+
+  memset(buttons_state, 0, sizeof(buttons_state));
 
   AM_unloadPics();
   automapactive = false;
@@ -732,19 +762,6 @@ static void AM_maxOutWindowScale(void)
   AM_activateNewScale();
 }
 
-enum
-{
-  PAN_UP,
-  PAN_DOWN,
-  PAN_LEFT,
-  PAN_RIGHT,
-  ZOOM_IN,
-  ZOOM_OUT,
-  STATE_NUM
-};
-
-static int buttons_state[STATE_NUM] = { 0 };
-
 //
 // AM_Responder()
 //
@@ -779,7 +796,7 @@ boolean AM_Responder
 
   if (!automapactive)
   {
-    if (M_InputActivated(input_map))
+    if (M_InputActivated(input_map) && !WS_Override())
     {
       AM_Start ();
       viewactive = false;
@@ -793,28 +810,28 @@ boolean AM_Responder
     rc = true;
                                                                 // phares
     if (M_InputActivated(input_map_right))                      //    |
-      if (!followplayer)                                        //    V
+      if (!followplayer && !WS_HoldOverride())                  //    V
         buttons_state[PAN_RIGHT] = 1;
       else
         rc = false;
     else if (M_InputActivated(input_map_left))
-      if (!followplayer)
+      if (!followplayer && !WS_HoldOverride())
         buttons_state[PAN_LEFT] = 1;
       else
         rc = false;
     else if (M_InputActivated(input_map_up))
-      if (!followplayer)
+      if (!followplayer && !WS_HoldOverride())
         buttons_state[PAN_UP] = 1;
       else
         rc = false;
     else if (M_InputActivated(input_map_down))
-      if (!followplayer)
+      if (!followplayer && !WS_HoldOverride())
         buttons_state[PAN_DOWN] = 1;
       else
         rc = false;
     else if (M_InputActivated(input_map_zoomout))
     {
-      if (ev->type == ev_mouseb_down && M_IsMouseWheel(ev->data1))
+      if (ev->type == ev_mouseb_down && M_IsMouseWheel(ev->data1.i))
       {
         mousewheelzoom = true;
         mtof_zoommul = m_zoomout_mouse;
@@ -825,7 +842,7 @@ boolean AM_Responder
     }
     else if (M_InputActivated(input_map_zoomin))
     {
-      if (ev->type == ev_mouseb_down && M_IsMouseWheel(ev->data1))
+      if (ev->type == ev_mouseb_down && M_IsMouseWheel(ev->data1.i))
       {
         mousewheelzoom = true;
         mtof_zoommul = m_zoomin_mouse;
@@ -836,10 +853,16 @@ boolean AM_Responder
     }
     else if (M_InputActivated(input_map))
     {
-      bigstate = 0;
-      viewactive = true;
-      memset(buttons_state, 0, sizeof(buttons_state));
-      AM_Stop ();
+      if (!WS_Override())
+      {
+        bigstate = 0;
+        viewactive = true;
+        AM_Stop ();
+      }
+      else
+      {
+        rc = false;
+      }
     }
     else if (M_InputActivated(input_map_gobig))
     {
@@ -1030,11 +1053,11 @@ static void AM_doFollowPlayer(void)
 // pointer. Allows map inspection without moving player to the location.
 //
 
-int map_point_coordinates;
+static boolean map_point_coord;
 
 void AM_Coordinates(const mobj_t *mo, fixed_t *x, fixed_t *y, fixed_t *z)
 {
-  *z = followplayer || !map_point_coordinates || !automapactive ? *x = mo->x, *y = mo->y, mo->z :
+  *z = followplayer || !map_point_coord || !automapactive ? *x = mo->x, *y = mo->y, mo->z :
     R_PointInSubsector(*x = (m_x+m_w/2) << FRACTOMAPBITS, *y = (m_y+m_h/2) << FRACTOMAPBITS)->sector->floorheight;
 }
 
@@ -1489,19 +1512,19 @@ static void AM_drawGrid(int color)
     // [crispy] moved here
     ml.a.y = m_y;
     ml.b.y = m_y+m_h;
-    if (automaprotate)
+    if (automaprotate || ADJUST_ASPECT_RATIO)
     {
       ml.a.y -= m_w / 2;
       ml.b.y += m_w / 2;
-      AM_rotatePoint(&ml.a);
-      AM_rotatePoint(&ml.b);
     }
+    AM_transformPoint(&ml.a);
+    AM_transformPoint(&ml.b);
     AM_drawMline(&ml, color);
   }
 
   // Figure out start of horizontal gridlines
   start = m_y;
-  if (automaprotate)
+  if (automaprotate || ADJUST_ASPECT_RATIO)
   {
     start -= m_w / 2;
   }
@@ -1510,7 +1533,7 @@ static void AM_drawGrid(int color)
     start += // (MAPBLOCKUNITS<<FRACBITS)
       - ((start-(bmaporgy>>FRACTOMAPBITS))%gridsize);
   end = m_y + m_h;
-  if (automaprotate)
+  if (automaprotate || ADJUST_ASPECT_RATIO)
   {
     end += m_w / 2;
   }
@@ -1527,9 +1550,9 @@ static void AM_drawGrid(int color)
     {
       ml.a.x -= m_h / 2;
       ml.b.x += m_h / 2;
-      AM_rotatePoint(&ml.a);
-      AM_rotatePoint(&ml.b);
     }
+    AM_transformPoint(&ml.a);
+    AM_transformPoint(&ml.b);
     AM_drawMline(&ml, color);
   }
 }
@@ -1596,6 +1619,18 @@ static int AM_DoorColor(int type)
 // jff 4/3/98 changed mapcolor_xxxx=0 as control to disable feature
 // jff 4/3/98 changed mapcolor_xxxx=-1 to disable drawing line completely
 //
+
+#define M_ARRAY_INIT_CAPACITY 500
+#include "m_array.h"
+
+typedef struct
+{
+  mline_t l;
+  int color;
+} am_line_t;
+
+static am_line_t *lines_1S = NULL;
+
 static void AM_drawWalls(void)
 {
   int i;
@@ -1610,11 +1645,8 @@ static void AM_drawWalls(void)
     l.a.y = lines[i].v1->y >> FRACTOMAPBITS;
     l.b.x = lines[i].v2->x >> FRACTOMAPBITS;
     l.b.y = lines[i].v2->y >> FRACTOMAPBITS;
-    if (automaprotate)
-    {
-        AM_rotatePoint(&l.a);
-        AM_rotatePoint(&l.b);
-    }
+    AM_transformPoint(&l.a);
+    AM_transformPoint(&l.b);
     // if line has been seen or IDDT has been used
     if (ddt_cheating || (lines[i].flags & ML_MAPPED))
     {
@@ -1677,23 +1709,36 @@ static void AM_drawWalls(void)
 
       if (!lines[i].backsector)
       {
+        if (mapcolor_exit && P_IsDeathExit(lines[i].frontsector))
+        {
+          array_push(lines_1S, ((am_line_t){l, keyed_door_flash ? mapcolor_grid : mapcolor_exit}));
+        }
         // jff 1/10/98 add new color for 1S secret sector boundary
-        if (mapcolor_secr && //jff 4/3/98 0 is disable
+        else if (mapcolor_secr && //jff 4/3/98 0 is disable
             (
              !map_secret_after &&
              P_IsSecret(lines[i].frontsector)
             )
           )
-          AM_drawMline(&l, mapcolor_secr); // line bounding secret sector
+        {
+          // line bounding secret sector
+          array_push(lines_1S, ((am_line_t){l, mapcolor_secr}));
+        }
         else if (mapcolor_revsecr &&
             (
              P_WasSecret(lines[i].frontsector) &&
              !P_IsSecret(lines[i].frontsector)
             )
           )
-          AM_drawMline(&l, mapcolor_revsecr); // line bounding revealed secret sector
+        {
+          // line bounding revealed secret sector
+          array_push(lines_1S, ((am_line_t){l, mapcolor_revsecr}));
+        }
         else                               //jff 2/16/98 fixed bug
-          AM_drawMline(&l, mapcolor_wall); // special was cleared
+        {
+          // special was cleared
+          array_push(lines_1S, ((am_line_t){l, mapcolor_wall}));
+        }
       }
       else
       {
@@ -1721,6 +1766,13 @@ static void AM_drawWalls(void)
         {
           AM_drawMline(&l, mapcolor_clsd);      // non-secret closed door
         } //jff 1/6/98 show secret sector 2S lines
+        else if (mapcolor_exit &&
+            (P_IsDeathExit(lines[i].frontsector) ||
+             P_IsDeathExit(lines[i].backsector))
+        )
+        {
+          AM_drawMline(&l, keyed_door_flash ? mapcolor_grid : mapcolor_exit);
+        }
         else if
         (
             mapcolor_secr && //jff 2/16/98 fixed bug
@@ -1784,6 +1836,12 @@ static void AM_drawWalls(void)
       }
     }
   }
+
+  for (int i = 0; i < array_size(lines_1S); ++i)
+  {
+    AM_drawMline(&lines_1S[i].l, lines_1S[i].color);
+  }
+  array_clear(lines_1S);
 }
 
 //
@@ -1817,26 +1875,36 @@ static void AM_rotate
 
 // [crispy] rotate point around map center
 // adapted from prboom-plus/src/am_map.c:898-920
-static void AM_rotatePoint(mpoint_t *pt)
+// [Woof!] Also, scale y coordinate of point for square aspect ratio
+static void AM_transformPoint(mpoint_t *pt)
 {
-  int64_t tmpx;
-  // [crispy] smooth automap rotation
-  angle_t smoothangle = followplayer ? ANG90 - viewangle : mapangle;
+  if (automaprotate)
+  {
+    int64_t tmpx;
+    // [crispy] smooth automap rotation
+    angle_t smoothangle = followplayer ? ANG90 - viewangle : mapangle;
 
-  pt->x -= mapcenter.x;
-  pt->y -= mapcenter.y;
+    pt->x -= mapcenter.x;
+    pt->y -= mapcenter.y;
 
-  smoothangle >>= ANGLETOFINESHIFT;
+    smoothangle >>= ANGLETOFINESHIFT;
 
-  tmpx = (int64_t)FixedMul(pt->x, finecosine[smoothangle])
-       - (int64_t)FixedMul(pt->y, finesine[smoothangle])
-       + mapcenter.x;
+    tmpx = (int64_t)FixedMul(pt->x, finecosine[smoothangle])
+        - (int64_t)FixedMul(pt->y, finesine[smoothangle])
+        + mapcenter.x;
 
-  pt->y = (int64_t)FixedMul(pt->x, finesine[smoothangle])
-        + (int64_t)FixedMul(pt->y, finecosine[smoothangle])
-        + mapcenter.y;
+    pt->y = (int64_t)FixedMul(pt->x, finesine[smoothangle])
+          + (int64_t)FixedMul(pt->y, finecosine[smoothangle])
+          + mapcenter.y;
 
-  pt->x = tmpx;
+    pt->x = tmpx;
+  }
+  if (ADJUST_ASPECT_RATIO)
+  {
+    int64_t diff = pt->y - mapcenter.y;
+    diff = 5 * diff / 6;
+    pt->y = mapcenter.y + diff;
+  }
 }
 
 //
@@ -1880,6 +1948,9 @@ static void AM_drawLineCharacter
     if (angle)
       AM_rotate(&l.a.x, &l.a.y, angle);
 
+    if (ADJUST_ASPECT_RATIO)
+      l.a.y = 5 * l.a.y / 6;
+
     l.a.x += x;
     l.a.y += y;
 
@@ -1895,6 +1966,9 @@ static void AM_drawLineCharacter
     if (angle)
       AM_rotate(&l.b.x, &l.b.y, angle);
 
+    if (ADJUST_ASPECT_RATIO)
+      l.b.y = 5 * l.b.y / 6;
+
     l.b.x += x;
     l.b.y += y;
 
@@ -1935,10 +2009,7 @@ static void AM_drawPlayers(void)
         pt.x = plr->mo->x >> FRACTOMAPBITS;
         pt.y = plr->mo->y >> FRACTOMAPBITS;
     }
-    if (automaprotate)
-    {
-        AM_rotatePoint(&pt);
-    }
+    AM_transformPoint(&pt);
 
     if (ddt_cheating)
       AM_drawLineCharacter
@@ -1984,7 +2055,7 @@ static void AM_drawPlayers(void)
       color = mapcolor_plyr[their_color];   //jff 1/6/98 use default color
 
     // [crispy] interpolate other player arrows
-    if (uncapped && leveltime > oldleveltime)
+    if (uncapped && leveltime > oldleveltime && p->mo->interp)
     {
         pt.x = LerpFixed(p->mo->oldx, p->mo->x) >> FRACTOMAPBITS;
         pt.y = LerpFixed(p->mo->oldy, p->mo->y) >> FRACTOMAPBITS;
@@ -1995,9 +2066,9 @@ static void AM_drawPlayers(void)
         pt.y = p->mo->y >> FRACTOMAPBITS;
     }
 
+    AM_transformPoint(&pt);
     if (automaprotate)
     {
-      AM_rotatePoint(&pt);
       smoothangle = p->mo->angle;
     }
     else
@@ -2048,7 +2119,7 @@ static void AM_drawThings
       }
 
       // [crispy] interpolate thing triangles movement
-      if (leveltime > oldleveltime)
+      if (uncapped && leveltime > oldleveltime && t->interp)
       {
         pt.x = LerpFixed(t->oldx, t->x) >> FRACTOMAPBITS;
         pt.y = LerpFixed(t->oldy, t->y) >> FRACTOMAPBITS;
@@ -2058,10 +2129,7 @@ static void AM_drawThings
         pt.x = t->x >> FRACTOMAPBITS;
         pt.y = t->y >> FRACTOMAPBITS;
       }
-      if (automaprotate)
-      {
-        AM_rotatePoint(&pt);
-      }
+      AM_transformPoint(&pt);
 
       //jff 1/5/98 case over doomednum of thing being drawn
       if (mapcolor_rkey || mapcolor_ykey || mapcolor_bkey)
@@ -2165,10 +2233,7 @@ static void AM_drawMarks(void)
 	// [crispy] center marks around player
 	pt.x = markpoints[i].x;
 	pt.y = markpoints[i].y;
-	if (automaprotate)
-	{
-	  AM_rotatePoint(&pt);
-	}
+	AM_transformPoint(&pt);
 	fx = CXMTOF(pt.x);
 	fy = CYMTOF(pt.y);
 
@@ -2234,25 +2299,25 @@ void AM_Drawer (void)
     AM_changeWindowLoc();
   }
 
-  // [crispy] required for AM_rotatePoint()
-  if (automaprotate)
+  // [crispy/Woof!] required for AM_transformPoint()
+  if (automaprotate || ADJUST_ASPECT_RATIO)
   {
     mapcenter.x = m_x + m_w / 2;
     mapcenter.y = m_y + m_h / 2;
     // [crispy] keep the map static if not following the player
-    if (followplayer)
+    if (automaprotate && followplayer)
     {
       mapangle = ANG90 - plr->mo->angle;
     }
   }
 
-  if (!automapoverlay)
+  if (automapoverlay == AM_OVERLAY_OFF)
   {
     AM_clearFB(mapcolor_back);       //jff 1/5/98 background default color
     pspr_interp = false;
   }
   // [Alaux] Dark automap overlay
-  else if (automapoverlay == AM_OVERLAY_DARK && !M_MenuIsShaded())
+  else if (automapoverlay == AM_OVERLAY_DARK && !MN_MenuIsShaded())
     V_ShadeScreen();
 
   if (automap_grid)                  // killough 2/28/98: change var name
@@ -2266,62 +2331,137 @@ void AM_Drawer (void)
   AM_drawMarks();
 }
 
-int mapcolor_preset;
+typedef enum {
+  AM_PRESET_VANILLA,
+  AM_PRESET_CRISPY,
+  AM_PRESET_BOOM,
+  AM_PRESET_ZDOOM,
+  NUM_AM_PRESETS
+} am_preset_t;
+
+static am_preset_t mapcolor_preset;
 
 void AM_ColorPreset(void)
 {
   struct
   {
     int *var;
-    int color[3]; // Vanilla Doom, Boom, ZDoom
+    int color[NUM_AM_PRESETS]; // Vanilla Doom, Crispy, Boom, ZDoom
   } mapcolors[] =
-  {                                       // ZDoom CVAR name
-    {&mapcolor_back,    {  0, 247, 139}}, // am_backcolor
-    {&mapcolor_grid,    {104, 104,  70}}, // am_gridcolor
-    {&mapcolor_wall,    {176,  23, 239}}, // am_wallcolor
-    {&mapcolor_fchg,    { 64,  55, 135}}, // am_fdwallcolor
-    {&mapcolor_cchg,    {231, 215,  76}}, // am_cdwallcolor
-    {&mapcolor_clsd,    {  0, 208,   0}},
-    {&mapcolor_rkey,    {176, 175, 176}}, // P_GetMapColorForLock()
-    {&mapcolor_bkey,    {200, 204, 200}}, // P_GetMapColorForLock()
-    {&mapcolor_ykey,    {231, 231, 231}}, // P_GetMapColorForLock()
-    {&mapcolor_rdor,    {176, 175, 176}}, // P_GetMapColorForLock()
-    {&mapcolor_bdor,    {200, 204, 200}}, // P_GetMapColorForLock()
-    {&mapcolor_ydor,    {231, 231, 231}}, // P_GetMapColorForLock()
-    {&mapcolor_tele,    {  0, 119, 200}}, // am_intralevelcolor
-    {&mapcolor_secr,    {  0, 252, 251}}, // am_unexploredsecretcolor
-    {&mapcolor_revsecr, {  0, 112, 251}}, // am_secretsectorcolor
-    {&mapcolor_exit,    {  0, 208, 176}}, // am_interlevelcolor
-    {&mapcolor_unsn,    { 99, 104, 100}}, // am_notseencolor
-    {&mapcolor_flat,    { 97,  88,  95}}, // am_tswallcolor
-    {&mapcolor_sprt,    {112, 112,   4}}, // am_thingcolor
-    {&mapcolor_hair,    { 96, 208,  97}}, // am_xhaircolor
-    {&mapcolor_sngl,    {209, 208, 209}}, // am_yourcolor
-    {&mapcolor_plyr[0], {112, 112, 112}},
-    {&mapcolor_plyr[1], { 88,  88,  88}},
-    {&mapcolor_plyr[2], { 64,  64,  64}},
-    {&mapcolor_plyr[3], {176, 176, 176}},
-    {&mapcolor_frnd,    {252, 252,   4}}, // am_thingcolor_friend
-    {&mapcolor_enemy,   {112, 177,   4}}, // am_thingcolor_monster
-    {&mapcolor_item,    {112, 231,   4}}, // am_thingcolor_item
+  {                                            // ZDoom CVAR name
+    {&mapcolor_back,    {  0,   0, 247, 139}}, // am_backcolor
+    {&mapcolor_grid,    {104, 104, 104,  70}}, // am_gridcolor
+    {&mapcolor_wall,    {176, 180,  23, 239}}, // am_wallcolor
+    {&mapcolor_fchg,    { 64,  70,  55, 135}}, // am_fdwallcolor
+    {&mapcolor_cchg,    {231, 163, 215,  76}}, // am_cdwallcolor
+    {&mapcolor_clsd,    {  0,   0, 208,   0}},
+    {&mapcolor_rkey,    {176, 176, 175, 176}}, // P_GetMapColorForLock()
+    {&mapcolor_bkey,    {200, 200, 204, 200}}, // P_GetMapColorForLock()
+    {&mapcolor_ykey,    {231, 231, 231, 231}}, // P_GetMapColorForLock()
+    {&mapcolor_rdor,    {176, 174, 175, 176}}, // P_GetMapColorForLock()
+    {&mapcolor_bdor,    {200, 200, 204, 200}}, // P_GetMapColorForLock()
+    {&mapcolor_ydor,    {231, 229, 231, 231}}, // P_GetMapColorForLock()
+    {&mapcolor_tele,    {  0, 120, 119, 200}}, // am_intralevelcolor
+    {&mapcolor_secr,    {  0,  -1, 252, 251}}, // am_unexploredsecretcolor
+    {&mapcolor_revsecr, {  0,  -1, 112, 251}}, // am_secretsectorcolor
+    {&mapcolor_exit,    {  0, 209, 208, 176}}, // am_interlevelcolor
+    {&mapcolor_unsn,    { 99,  99, 104, 100}}, // am_notseencolor
+    {&mapcolor_flat,    { 96,  96,  88,  95}}, // am_tswallcolor
+    {&mapcolor_sprt,    {112, 112, 112,   4}}, // am_thingcolor
+    {&mapcolor_hair,    { 96,  96, 208,  97}}, // am_xhaircolor
+    {&mapcolor_sngl,    {209, 209, 208, 209}}, // am_yourcolor
+    {&mapcolor_plyr[0], {112, 112, 112, 112}},
+    {&mapcolor_plyr[1], { 96,  96,  88,  88}},
+    {&mapcolor_plyr[2], { 64,  64,  64,  64}},
+    {&mapcolor_plyr[3], {176, 176, 176, 176}},
+    {&mapcolor_frnd,    {252, 252, 252,   4}}, // am_thingcolor_friend
+    {&mapcolor_enemy,   {112, 176, 177,   4}}, // am_thingcolor_monster
+    {&mapcolor_item,    {112, 231, 231,   4}}, // am_thingcolor_item
 
-    {&hudcolor_titl,    {CR_NONE, CR_GOLD, CR_GRAY}}, // DrawAutomapHUD()
-
-    {NULL,              {  0,   0,   0}},
+    {&hudcolor_titl,    {CR_NONE, CR_GOLD, CR_GOLD, CR_GRAY}}, // DrawAutomapHUD()
   };
 
-  int i;
-
-  for (i = 0; mapcolors[i].var; i++)
+  for (int i = 0; i < arrlen(mapcolors); i++)
   {
     *mapcolors[i].var = mapcolors[i].color[mapcolor_preset];
   }
 
-  // [FG] immediately apply changes if the automap is visible through the menu
-  if (automapactive && menu_backdrop != MENU_BG_TEXTURE)
+  // [crispy] Make secret wall colors independent from PLAYPAL color indexes
+  if (mapcolor_preset == AM_PRESET_CRISPY)
   {
-    HU_Start();
+    byte *playpal = W_CacheLumpName("PLAYPAL", PU_CACHE);
+    mapcolor_secr = I_GetNearestColor(playpal, 255, 0, 255);
+    mapcolor_revsecr = I_GetNearestColor(playpal, 119, 255, 111);
   }
+
+  ST_ResetTitle();
+}
+
+void AM_BindAutomapVariables(void)
+{
+  M_BindBool("followplayer", &followplayer, NULL, true, ss_auto, wad_no,
+             "Automap follows the player");
+  M_BindNum("automapoverlay", &automapoverlay, NULL, AM_OVERLAY_OFF,
+            AM_OVERLAY_OFF, AM_OVERLAY_DARK, ss_auto, wad_no,
+            "Automap overlay mode (0 = Off; 1 = On; 2 = Dark)");
+  M_BindBool("automaprotate", &automaprotate, NULL, false, ss_auto, wad_no,
+             "Automap rotation");
+
+  M_BindBool("map_point_coord", &map_point_coord, NULL, true, ss_none, wad_no,
+             "Show automap pointer coordinates in non-follow mode");
+  M_BindBool("map_secret_after", &map_secret_after, NULL, false, ss_auto, wad_no,
+             "Don't highlight secret sectors on the automap before they're revealed");
+  M_BindNum("map_keyed_door", &map_keyed_door, NULL,
+            MAP_KEYED_DOOR_COLOR, MAP_KEYED_DOOR_OFF, MAP_KEYED_DOOR_FLASH,
+            ss_auto, wad_no,
+            "Color key-locked doors on the automap (1 = Static; 2 = Flashing)");
+  M_BindBool("map_smooth_lines", &map_smooth_lines, NULL, true, ss_none,
+             wad_no, "Smooth automap lines");
+
+  M_BindNum("mapcolor_preset", &mapcolor_preset, NULL, AM_PRESET_BOOM,
+            AM_PRESET_VANILLA, AM_PRESET_ZDOOM, ss_auto, wad_no,
+            "Automap color preset (0 = Vanilla Doom; 1 = Crispy Doom; 2 = Boom; 3 = ZDoom)");
+
+  M_BindBool("automapsquareaspect", &automapsquareaspect, NULL, true, ss_none, wad_no,
+             "Use square aspect ratio in automap");
+
+#define BIND_CR(name, v, help) \
+  M_BindNum(#name, &name, NULL, (v), 0, 255, ss_none, wad_yes, help)
+
+  BIND_CR(mapcolor_back, 247, "Color used for the automap background");
+  BIND_CR(mapcolor_grid, 104, "Color used for grid lines");
+  BIND_CR(mapcolor_wall, 23, "Color used for one-sided walls");
+  BIND_CR(mapcolor_fchg, 55, "Color used for lines with floor height changes");
+  BIND_CR(mapcolor_cchg, 215, "Color used for lines with ceiling height changes");
+  BIND_CR(mapcolor_clsd, 208, "Color used for lines denoting closed doors, objects");
+  BIND_CR(mapcolor_rkey, 175, "Color used for red-key sprites");
+  BIND_CR(mapcolor_bkey, 204, "Color used for blue-key sprites");
+  BIND_CR(mapcolor_ykey, 231, "Color used for yellow-key sprites");
+  BIND_CR(mapcolor_rdor, 175, "Color used for closed red doors");
+  BIND_CR(mapcolor_bdor, 204, "Color used for closed blue doors");
+  BIND_CR(mapcolor_ydor, 231, "Color used for closed yellow doors");
+  BIND_CR(mapcolor_tele, 119, "Color used for teleporter lines");
+  BIND_CR(mapcolor_secr, 252, "Color used for lines around secret sectors");
+  BIND_CR(mapcolor_revsecr, 112, "Color used for lines around revealed secret sectors");
+  BIND_CR(mapcolor_exit, 0, "Color used for exit lines");
+  BIND_CR(mapcolor_unsn, 104, "Color used for lines not seen without computer map");
+  BIND_CR(mapcolor_flat, 88, "Color used for lines with no height changes");
+  BIND_CR(mapcolor_sprt, 112, "Color used for things");
+  BIND_CR(mapcolor_hair, 208, "Color used for the automap pointer/crosshair");
+  BIND_CR(mapcolor_sngl, 208, "Color used for the player's arrow (single-player only)");
+
+#define BIND_PLR_CR(num, v, help)                                           \
+  M_BindNum("mapcolor_ply"#num, &mapcolor_plyr[(num)-1], NULL, (v), 0, 255, \
+            ss_none, wad_yes, help)
+
+  BIND_PLR_CR(1, 112, "Color used for the green player's arrow");
+  BIND_PLR_CR(2, 88, "Color used for the gray player's arrow");
+  BIND_PLR_CR(3, 64, "Color used for the brown player's arrow");
+  BIND_PLR_CR(4, 176, "Color used for the red player's arrow");
+
+  BIND_CR(mapcolor_frnd, 252, "Color used for friends");
+  BIND_CR(mapcolor_enemy, 177, "Color used for enemies");
+  BIND_CR(mapcolor_item, 231, "Color used for countable items");
 }
 
 //----------------------------------------------------------------------------
