@@ -28,22 +28,32 @@
 #include "m_misc.h"
 
 static const iwad_t iwads[] = {
-    {"doom2.wad",     doom2,      commercial,   vanilla,  "Doom II"                   },
-    {"plutonia.wad",  pack_plut,  commercial,   vanilla,
-     "Final Doom: Plutonia Experiment"                                                },
-    {"tnt.wad",       pack_tnt,   commercial,   vanilla,  "Final Doom: TNT: Evilution"},
- // "doom.wad" may be retail or registered
-    {"doom.wad",      doom,       indetermined, vanilla,  "Doom"                      },
-    {"doom1.wad",     doom,       indetermined, vanilla,  "Doom Shareware"            },
-    {"doom2f.wad",    doom2,      commercial,   vanilla,  "Doom II: L'Enfer sur Terre"},
-    {"chex.wad",      pack_chex,  retail,       vanilla,  "Chex Quest"                },
-    {"hacx.wad",      pack_hacx,  commercial,   vanilla,  "Hacx"                      },
-    {"freedoom2.wad", doom2,      commercial,   freedoom, "Freedoom: Phase 2"         },
-    {"freedoom1.wad", doom,       retail,       freedoom, "Freedoom: Phase 1"         },
-    {"freedm.wad",    doom2,      commercial,   freedoom, "FreeDM"                    },
-    {"rekkrsa.wad",   pack_rekkr, retail,       vanilla,  "REKKR"                     },
-    {"rekkrsl.wad",   pack_rekkr, retail,       vanilla,  "REKKR: Sunken Land"        },
-    {"miniwad.wad",   doom2,      commercial,   miniwad,  "miniwad"                   }
+    {"doom2.wad",     doom2,      commercial,   "DOOM II: Hell on Earth"         },
+    {"plutonia.wad",  pack_plut,  commercial,   "Final DOOM: Plutonia Experiment"},
+    {"tnt.wad",       pack_tnt,   commercial,   "Final DOOM: TNT - Evilution"    },
+    // "doom.wad" may be retail or registered
+    {"doom.wad",      doom,       indetermined, "DOOM"                           },
+    {"doom.wad",      doom,       registered,   "DOOM Registered"                },
+    {"doom.wad",      doom,       retail,       "The Ultimate DOOM"              },
+    // "doomu.wad" alias to allow retail wad to coexist with registered in the same folder
+    {"doomu.wad",     doom,       retail,       "The Ultimate DOOM"              },
+    {"doom1.wad",     doom,       shareware,    "DOOM Shareware"                 },
+    {"doom2f.wad",    doom2,      commercial,   "DOOM II: L'Enfer sur Terre"     },
+    {"freedoom2.wad", doom2,      commercial,   "Freedoom: Phase 2"              },
+    {"freedoom1.wad", doom,       retail,       "Freedoom: Phase 1"              },
+    {"freedm.wad",    doom2,      commercial,   "FreeDM"                         },
+    {"chex.wad",      pack_chex,  retail,       "Chex Quest"                     },
+    {"hacx.wad",      pack_hacx,  commercial,   "HACX: Twitch n' Kill"           },
+    {"rekkrsa.wad",   pack_rekkr, retail,       "REKKR"                          },
+    {"rekkrsl.wad",   pack_rekkr, retail,       "REKKR: Sunken Land"             },
+};
+
+static const char *const gamemode_str[] = {
+    "Shareware mode",
+    "Registered mode",
+    "Commercial mode",
+    "Retail mode",
+    "Unknown mode"
 };
 
 // "128 IWAD search directories should be enough for anybody".
@@ -222,6 +232,13 @@ static registry_value_t root_path_keys[] =
         SOFTWARE_KEY "\\GOG.com\\Games\\1426071866",
         "PATH"
     },
+
+    // DOOM + DOOM II
+    {
+        HKEY_LOCAL_MACHINE,
+        SOFTWARE_KEY "\\GOG.com\\Games\\1413291984",
+        "PATH"
+    },
 };
 
 // Subdirectories of the above install path, where IWADs are installed.
@@ -245,16 +262,18 @@ static registry_value_t steam_install_location =
 
 static char *steam_install_subdirs[] =
 {
-    "steamapps\\common\\doom 2\\base", "steamapps\\common\\final doom\\base",
-    "steamapps\\common\\ultimate doom\\base",
-
-    // From Doom 3: BFG Edition:
-
+    "steamapps\\common\\Doom 2\\base",
+    "steamapps\\common\\Doom 2\\rerelease\\DOOM II_Data\\StreamingAssets",
+    "steamapps\\common\\Doom 2\\finaldoombase",
+    "steamapps\\common\\Doom 2\\masterbase\\doom2",
+    "steamapps\\common\\Final Doom\\base",
+    "steamapps\\common\\Master Levels of Doom\\doom2",
+    "steamapps\\common\\Ultimate Doom\\base",
+    "steamapps\\common\\Ultimate Doom\\base\\doom2",
+    "steamapps\\common\\Ultimate Doom\\rerelease",
+    "steamapps\\common\\Ultimate Doom\\rerelease\\DOOM_Data\\StreamingAssets",
     "steamapps\\common\\DOOM 3 BFG Edition\\base\\wads",
-
-    "steamapps\\common\\ultimate doom\\rerelease\\DOOM_Data\\StreamingAssets",
-    "steamapps\\common\\doom 2\\rerelease\\DOOM II_Data\\StreamingAssets",
-    "steamapps\\common\\doom 2\\finaldoombase"
+    "steamapps\\common\\DOOMEternal\\base\\classicwads",
 };
 
 static char *GetRegistryString(registry_value_t *reg_val)
@@ -361,7 +380,7 @@ static void CheckInstallRootPaths(void)
         for (j = 0; j < arrlen(root_path_subdirs); ++j)
         {
             subpath = M_StringJoin(install_path, DIR_SEPARATOR_S,
-                                   root_path_subdirs[j], NULL);
+                                   root_path_subdirs[j]);
             AddIWADDir(subpath);
         }
 
@@ -387,7 +406,7 @@ static void CheckSteamEdition(void)
     for (i = 0; i < arrlen(steam_install_subdirs); ++i)
     {
         subpath = M_StringJoin(install_path, DIR_SEPARATOR_S,
-                               steam_install_subdirs[i], NULL);
+                               steam_install_subdirs[i]);
 
         AddIWADDir(subpath);
     }
@@ -443,7 +462,7 @@ static void AddIWADPath(const char *path, const char *suffix)
             // as another iwad dir
             *p = '\0';
 
-            AddIWADDir(M_StringJoin(left, suffix, NULL));
+            AddIWADDir(M_StringJoin(left, suffix));
             left = p + 1;
         }
         else
@@ -452,7 +471,7 @@ static void AddIWADPath(const char *path, const char *suffix)
         }
     }
 
-    AddIWADDir(M_StringJoin(left, suffix, NULL));
+    AddIWADDir(M_StringJoin(left, suffix));
 
     free(dup_path);
 }
@@ -483,14 +502,14 @@ static void AddXdgDirs(void)
             homedir = "/";
         }
 
-        tmp_env = M_StringJoin(homedir, "/.local/share", NULL);
+        tmp_env = M_StringJoin(homedir, "/.local/share");
         env = tmp_env;
     }
 
     // We support $XDG_DATA_HOME/games/doom (which will usually be
     // ~/.local/share/games/doom) as a user-writeable extension to
     // the usual /usr/share/games/doom location.
-    AddIWADDir(M_StringJoin(env, "/games/doom", NULL));
+    AddIWADDir(M_StringJoin(env, "/games/doom"));
     free(tmp_env);
 
     // Quote:
@@ -534,7 +553,7 @@ static void AddSteamDirs(void)
     {
         homedir = "/";
     }
-    steampath = M_StringJoin(homedir, "/.steam/root/steamapps/common", NULL);
+    steampath = M_StringJoin(homedir, "/.steam/root/steamapps/common");
 
     AddIWADPath(steampath, "/Doom 2/base");
     AddIWADPath(steampath, "/Master Levels of Doom/doom2");
@@ -587,11 +606,6 @@ void BuildIWADDirList(void)
         AddIWADDir(env);
     }
 
-#if defined(WOOFDATADIR)
-    // [FG] Add a build-time configurable data directory
-    AddIWADDir(WOOFDATADIR);
-#endif
-
 #ifdef _WIN32
 
     // Search the registry and find where IWADs have been installed.
@@ -650,7 +664,7 @@ char *D_FindWADByName(const char *name)
 
         // Construct a string for the full path
 
-        path = M_StringJoin(iwad_dirs[i], DIR_SEPARATOR_S, name, NULL);
+        path = M_StringJoin(iwad_dirs[i], DIR_SEPARATOR_S, name);
 
         probe = M_FileCaseExists(path);
         if (probe != NULL)
@@ -666,21 +680,19 @@ char *D_FindWADByName(const char *name)
     return NULL;
 }
 
-static char *FindWithExtensions(const char *filename, ...)
+#define FindWithExtensions(filename, ...)                               \
+    FindWithExtensionsInternal(filename, (const char *[]){__VA_ARGS__}, \
+                               sizeof((const char *[]){__VA_ARGS__})    \
+                                   / sizeof(const char *))
+
+static char *FindWithExtensionsInternal(const char *filename,
+                                        const char *ext[], size_t n)
 {
     char *path = NULL;
-    va_list args;
 
-    va_start(args, filename);
-    while (true)
+    for (int i = 0; i < n; ++i)
     {
-        const char *arg = va_arg(args, const char *);
-        if (arg == NULL)
-        {
-            break;
-        }
-
-        char *s = M_StringJoin(filename, arg, NULL);
+        char *s = M_StringJoin(filename, ext[i]);
         path = D_FindWADByName(s);
         free(s);
         if (path != NULL)
@@ -688,7 +700,6 @@ static char *FindWithExtensions(const char *filename, ...)
             break;
         }
     }
-    va_end(args);
 
     return path;
 }
@@ -706,7 +717,7 @@ char *D_TryFindWADByName(const char *filename)
 
     if (!strrchr(M_BaseName(filename), '.'))
     {
-        result = FindWithExtensions(filename, ".wad", ".zip", ".lmp", NULL);
+        result = FindWithExtensions(filename, ".wad", ".zip", ".lmp");
     }
     else
     {
@@ -723,12 +734,23 @@ char *D_TryFindWADByName(const char *filename)
     }
 }
 
+char *D_FindLMPByName(const char *filename)
+{
+    if (!strrchr(M_BaseName(filename), '.'))
+    {
+        return FindWithExtensions(filename, ".lmp");
+    }
+    else
+    {
+        return D_FindWADByName(filename);
+    }
+}
+
 //
 // D_FindIWADFile
 //
 
-char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission,
-                     GameVariant_t *variant)
+char *D_FindIWADFile(void)
 {
     char *result;
 
@@ -773,23 +795,6 @@ char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission,
         }
     }
 
-    if (result)
-    {
-        int i;
-        const char *name = M_BaseName(result);
-
-        for (i = 0; i < arrlen(iwads); ++i)
-        {
-            if (!strcasecmp(name, iwads[i].name))
-            {
-                *mode = iwads[i].mode;
-                *mission = iwads[i].mission;
-                *variant = iwads[i].variant;
-                break;
-            }
-        }
-    }
-
     return result;
 }
 
@@ -837,3 +842,44 @@ const iwad_t **D_GetIwads(void)
 
     return result;
 }
+
+GameMission_t D_GetGameMissionByIWADName(const char *name)
+{
+    for (int i = 0; i < arrlen(iwads); ++i)
+    {
+        if (!strcasecmp(name, iwads[i].name))
+        {
+            return iwads[i].mission;
+        }
+    }
+
+    return none;
+}
+
+void D_GetModeAndMissionByIWADName(const char *name, GameMode_t *mode,
+                                   GameMission_t *mission)
+{
+    for (int i = 0; i < arrlen(iwads); ++i)
+    {
+        if (!strcasecmp(name, iwads[i].name))
+        {
+            *mode = iwads[i].mode;
+            *mission = iwads[i].mission;
+            break;
+        }
+    }
+}
+
+const char *D_GetIWADDescription(const char *name, GameMode_t mode,
+                                 GameMission_t mission)
+{
+    for (int i = 0; i < arrlen(iwads); ++i)
+    {
+        if (!strcasecmp(name, iwads[i].name) && mode == iwads[i].mode
+            && mission == iwads[i].mission)
+        {
+            return iwads[i].description;
+        }
+    }
+    return gamemode_str[mode];
+}
