@@ -43,13 +43,6 @@ typedef enum {
   none
 } GameMission_t;
 
-typedef enum
-{
-    vanilla,    // Vanilla Doom
-    freedoom,   // FreeDoom: Phase 1 + 2 and FreeDM
-    miniwad     // miniwad
-} GameVariant_t;
-
 // Identify language to use, software localization.
 typedef enum {
   english,
@@ -62,6 +55,7 @@ typedef enum {
 
 typedef enum
 {
+    exe_indetermined = -1,
     exe_doom_1_9,    // Doom 1.9: for shareware, registered and commercial
     exe_ultimate,    // Ultimate Doom (retail)
     exe_final,       // Final Doom
@@ -94,6 +88,7 @@ typedef enum
 // at the intermission screen, the game final animation, or a demo.
 
 typedef enum {
+  GS_NONE = -1,
   GS_LEVEL,
   GS_INTERMISSION,
   GS_FINALE,
@@ -181,6 +176,33 @@ typedef enum {
   NUMPOWERS,
 } powertype_t;
 
+typedef enum
+{
+    item_noitem = -1,
+    item_messageonly,
+    item_bluecard,
+    item_yellowcard,
+    item_redcard,
+    item_blueskull,
+    item_yellowskull,
+    item_redskull,
+    item_backpack,
+    item_healthbonus,
+    item_stimpack,
+    item_medikit,
+    item_soulsphere,
+    item_megasphere,
+    item_armorbonus,
+    item_greenarmor,
+    item_bluearmor,
+    item_areamap,
+    item_lightamp,
+    item_berserk,
+    item_invisibility,
+    item_radsuit,
+    item_invulnerability,
+} itemtype_t;
+
 // Power up durations (how many seconds till expiration).
 typedef enum {
   INVULNTICS  = (30*TICRATE),
@@ -194,7 +216,7 @@ typedef enum {
 // Used when resetting the defaults for every item in a Setup group.
 
 typedef enum {
-  ss_none,
+  ss_none = -1,
   ss_keys,
   ss_weap,
   ss_stat,
@@ -202,9 +224,19 @@ typedef enum {
   ss_enem,
   ss_gen,       // killough 10/98
   ss_comp,      // killough 10/98
+  ss_sfx,
+  ss_music,
+  ss_eq,
+  ss_padadv,
+  ss_gyro,
   ss_max
 } ss_types;
 
+typedef enum {
+  wad_no,
+  wad_yes
+} wad_allowed_t;
+
 // phares 3/20/98:
 //
 // Player friction is variable, based on controlling
