@@ -19,6 +19,7 @@
 #define __I_PRINT__
 
 #include "doomtype.h"
+#include "d_event.h"
 
 typedef enum
 {
@@ -30,7 +31,7 @@ typedef enum
     VB_MAX
 } verbosity_t;
 
-extern verbosity_t cfg_verbosity;
+#define VB_DEMO (gameaction == ga_playdemo ? VB_INFO : VB_DEBUG)
 
 int I_ConsoleStdout(void);
 
