@@ -35,6 +35,8 @@
 #include "m_fixed.h"
 #include "m_input.h"
 #include "m_misc.h"
+#include "mn_menu.h"
+#include "p_action.h"
 #include "p_inter.h"
 #include "p_map.h"
 #include "p_mobj.h"
@@ -42,12 +44,15 @@
 #include "p_spec.h" // SPECHITS
 #include "p_tick.h"
 #include "r_defs.h"
+#include "r_main.h"
 #include "r_state.h"
 #include "s_sound.h"
 #include "sounds.h"
+#include "st_widgets.h"
 #include "tables.h"
 #include "u_mapinfo.h"
 #include "w_wad.h"
+#include "ws_stuff.h"
 
 #define plyr (players+consoleplayer)     /* the console player */
 
@@ -104,6 +109,7 @@ static void cheat_reveal_item();
 static void cheat_autoaim();      // killough 7/19/98
 static void cheat_tst();
 static void cheat_showfps(); // [FG] FPS counter widget
+static void cheat_speed();
 
 //-----------------------------------------------------------------------------
 //
@@ -220,7 +226,7 @@ struct cheat_s cheat[] = {
   {"iddt",       "Map cheat",         not_dm,
    {cheat_ddt} },        // killough 2/07/98: moved from am_map.c
 
-  {"iddst",      NULL,                always,
+  {"iddst",      NULL,                not_dm,
    {cheat_reveal_secret} },
 
   {"iddkt",      NULL,                not_dm,
@@ -321,6 +327,9 @@ struct cheat_s cheat[] = {
   {"showfps",    NULL,                always,
    {cheat_showfps} },
 
+  {"speed",      NULL,                not_dm,
+   {cheat_speed} },
+
   {NULL}                 // end-of-list marker
 };
 
@@ -332,10 +341,14 @@ static void cheat_showfps()
   plyr->cheats ^= CF_SHOWFPS;
 }
 
+static void cheat_speed()
+{
+  speedometer = (speedometer + 1) % 4;
+}
+
 // killough 7/19/98: Autoaiming optional in beta emulation mode
 static void cheat_autoaim()
 {
-  extern int autoaim;
   displaymsg((autoaim=!autoaim) ?
     "Projectile autoaiming on" : 
     "Projectile autoaiming off");
@@ -417,7 +430,6 @@ static void cheat_god()
   {
     signed int an;
     mapthing_t mt = {0};
-    extern void P_SpawnPlayer (mapthing_t* mthing);
 
     P_MapStart();
     mt.x = plyr->mo->x >> FRACBITS;
@@ -429,7 +441,7 @@ static void cheat_god()
     // [crispy] spawn a teleport fog
     an = plyr->mo->angle >> ANGLETOFINESHIFT;
     P_SpawnMobj(plyr->mo->x+20*finecosine[an], plyr->mo->y+20*finesine[an], plyr->mo->z, MT_TFOG);
-    S_StartSound(plyr->mo, sfx_slop);
+    S_StartSoundEx(plyr->mo, sfx_slop);
     P_MapEnd();
 
     // Fix reviving as "zombie" if god mode was already enabled
@@ -530,7 +542,7 @@ static void cheat_fa()
   // You can't own weapons that aren't in the game // phares 02/27/98
   for (i=0;i<NUMWEAPONS;i++)
     if (!(((i == wp_plasma || i == wp_bfg) && gamemode == shareware) ||
-          (i == wp_supershotgun && !have_ssg)))
+          (i == wp_supershotgun && !ALLOW_SSG)))
       plyr->weaponowned[i] = true;
         
   for (i=0;i<NUMAMMO;i++)
@@ -599,10 +611,10 @@ static void cheat_clev0()
   int epsd, map;
   char *cur, *next;
 
-  cur = M_StringDuplicate(MAPNAME(gameepisode, gamemap));
+  cur = M_StringDuplicate(MapName(gameepisode, gamemap));
 
   G_GotoNextLevel(&epsd, &map);
-  next = MAPNAME(epsd, map);
+  next = MapName(epsd, map);
 
   if (W_CheckNumForName(next) != -1)
     displaymsg("Current: %s, Next: %s", cur, next);
@@ -645,7 +657,7 @@ static void cheat_clev(char *buf)
       epsd = 1;
     }
 
-    next = MAPNAME(epsd, map);
+    next = MapName(epsd, map);
 
     if (W_CheckNumForName(next) == -1)
     {
@@ -723,8 +735,6 @@ static void cheat_friction()
                                                "Variable Friction disabled");
 }
 
-extern const char *default_skill_strings[];
-
 static void cheat_skill0()
 {
   displaymsg("Skill: %s", default_skill_strings[gameskill + 1]);
@@ -769,7 +779,6 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
 
   int killcount=0;
   thinker_t *currentthinker=&thinkercap;
-  extern void A_PainDie(mobj_t *);
   // killough 7/20/98: kill friendly monsters only if no others to kill
   int mask = MF_FRIEND;
   P_MapStart();
@@ -812,6 +821,8 @@ static void cheat_spechits()
     plyr->cards[i] = true;
   }
 
+  P_MapStart();
+
   for (i = 0; i < numlines; i++)
   {
     if (lines[i].special)
@@ -831,6 +842,13 @@ static void cheat_spechits()
         case 126:
         case 174:
         case 195:
+        // [FG] do not trigger silent teleporters
+        case 207:
+        case 208:
+        case 209:
+        case 210:
+        case 268:
+        case 269:
         {
           continue;
         }
@@ -941,6 +959,8 @@ static void cheat_spechits()
     speciallines += EV_DoDoor(&dummy, doorOpen);
   }
 
+  P_MapEnd();
+
   displaymsg("%d Special Action%s Triggered", speciallines, speciallines == 1 ? "" : "s");
 }
 
@@ -948,7 +968,6 @@ static void cheat_spechits()
 // killough 3/26/98: emulate Doom better
 static void cheat_ddt()
 {
-  extern int ddt_cheating;
   if (automapactive)
     ddt_cheating = (ddt_cheating+1) % 3;
 }
@@ -993,7 +1012,6 @@ static void cheat_reveal_secret()
 static void cheat_cycle_mobj(mobj_t **last_mobj, int *last_count,
                              int flags, int alive)
 {
-  extern int init_thinkers_count;
   thinker_t *th, *start_th;
 
   // If the thinkers have been wiped, addresses are invalid
@@ -1055,7 +1073,6 @@ static void cheat_reveal_item()
 // killough 2/7/98: HOM autodetection
 static void cheat_hom()
 {
-  extern int autodetect_hom;           // Ty 03/27/98 - *not* externalized
   displaymsg((autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
     "HOM Detection Off");
 }
@@ -1089,7 +1106,7 @@ static void cheat_keyxx(int key)
 
 static void cheat_weap()
 {                                   // Ty 03/27/98 - *not* externalized
-  displaymsg(have_ssg ? // killough 2/28/98
+  displaymsg(ALLOW_SSG ? // killough 2/28/98
     "Weapon number 1-9" : "Weapon number 1-8");
 }
 
@@ -1097,7 +1114,7 @@ static void cheat_weapx(char *buf)
 {
   int w = *buf - '1';
 
-  if ((w==wp_supershotgun && !have_ssg) ||      // killough 2/28/98
+  if ((w==wp_supershotgun && !ALLOW_SSG) ||      // killough 2/28/98
       ((w==wp_bfg || w==wp_plasma) && gamemode==shareware))
     return;
 
@@ -1164,7 +1181,6 @@ static void cheat_pitch()
 
 static void cheat_nuke()
 {
-  extern int disable_nuke;
   displaymsg((disable_nuke = !disable_nuke) ? "Nukage Disabled" :
     "Nukage Enabled");
 }
@@ -1203,13 +1219,13 @@ boolean M_FindCheats(int key)
 
   if (argsleft)
     {
-      *arg++ = tolower(key);             // store key in arg buffer
+      *arg++ = M_ToLower(key);             // store key in arg buffer
       if (!--argsleft)                   // if last key in arg list,
         cheat[cht].func.s(argbuf);       // process the arg buffer
       return 1;                          // affirmative response
     }
 
-  key = tolower(key) - 'a';
+  key = M_ToLower(key) - 'a';
   if (key < 0 || key >= 32)              // ignore most non-alpha cheat letters
     {
       sr = 0;        // clear shift register
@@ -1225,7 +1241,7 @@ boolean M_FindCheats(int key)
           const char *p; // [FG] char!
           for (p=cheat[i].cheat; *p; p++)
             {
-              unsigned key = tolower(*p)-'a';  // convert to 0-31
+              unsigned key = M_ToLower(*p)-'a';  // convert to 0-31
               if (key >= 32)            // ignore most non-alpha cheat letters
                 continue;
               c = (c<<5) + key;         // shift key into code
@@ -1281,7 +1297,6 @@ static const struct {
   { input_idbeholdi, not_net|not_demo, {cheat_pw},       pw_invisibility },
   { input_idbeholdr, not_net|not_demo, {cheat_pw},       pw_ironfeet },
   { input_idbeholdl, not_dm,           {cheat_pw},       pw_infrared },
-  { input_idrate,    always,           {cheat_rate},     0 },
   { input_iddt,      not_dm,           {cheat_ddt},      0 },
   { input_notarget,  not_net|not_demo, {cheat_notarget}, 0 },
   { input_freeze,    not_net|not_demo, {cheat_freeze},   0 },
@@ -1292,9 +1307,15 @@ boolean M_CheatResponder(event_t *ev)
 {
   int i;
 
-  if (ev->type == ev_keydown && M_FindCheats(ev->data1))
+  if (strictmode && demorecording)
+    return false;
+
+  if (ev->type == ev_keydown && M_FindCheats(ev->data2.i))
     return true;
 
+  if (WS_Override())
+    return false;
+
   for (i = 0; i < arrlen(cheat_input); ++i)
   {
     if (M_InputActivated(cheat_input[i].input))
