@@ -329,10 +329,6 @@ static int P_IsUnderDamage(mobj_t *actor)
 static fixed_t xspeed[8] = {FRACUNIT,47000,0,-47000,-FRACUNIT,-47000,0,47000};
 static fixed_t yspeed[8] = {0,47000,FRACUNIT,47000,0,-47000,-FRACUNIT,-47000};
 
-// 1/11/98 killough: Limit removed on special lines crossed
-extern  line_t **spechit;          // New code -- killough
-extern  int    numspechit;
-
 static boolean P_Move(mobj_t *actor, boolean dropoff) // killough 9/12/98
 {
   fixed_t tryx, tryy, deltax, deltay;
@@ -444,24 +440,28 @@ static boolean P_Move(mobj_t *actor, boolean dropoff) // killough 9/12/98
         if (P_UseSpecialLine(actor, spechit[numspechit], 0, false))
 	  good |= spechit[numspechit] == blockline ? 1 : 2;
 
+      // There are checks elsewhere for numspechit == 0, so we don't want to
+      // leave numspechit == -1.
+      numspechit = 0;
+
       // [FG] compatibility maze here
       // Boom v2.01 and orig. Doom return "good"
       // Boom v2.02 and LxDoom return good && (P_Random(pr_trywalk)&3)
       // MBF plays even more games
 
-      if (demo_version < 202)
+      if (demo_version < DV_BOOM)
         return good;
-      if (demo_version < 203)
+      if (demo_version < DV_MBF)
         return good && (compatibility || (P_Random(pr_trywalk)&3)); //jff 8/13/98
       else
-      return good && (demo_version < 203 || comp[comp_doorstuck] ||
+      return good && (demo_version < DV_MBF || comp[comp_doorstuck] ||
 		      (P_Random(pr_opendoor) >= 230) ^ (good & 1));
     }
   else
     actor->flags &= ~MF_INFLOAT;
 
   // killough 11/98: fall more slowly, under gravity, if felldown==true
-  if (!(actor->flags & MF_FLOAT) && (!felldown || demo_version < 203))
+  if (!(actor->flags & MF_FLOAT) && (!felldown || demo_version < DV_MBF))
     actor->z = actor->floorz;
 
   return true;
@@ -496,7 +496,7 @@ static boolean P_SmartMove(mobj_t *actor)
       P_Random(pr_dropoff) < 235)
     dropoff = 2;
 
-  if (!P_Move(actor, dropoff))
+  if (!P_Move(actor, !!dropoff))
     return false;
 
   // killough 9/9/98: avoid crushing ceilings or other damaging areas
@@ -687,7 +687,7 @@ static void P_NewChaseDir(mobj_t *actor)
 
   actor->strafecount = 0;
 
-  if (demo_version >= 203)
+  if (demo_version >= DV_MBF)
   {
     if (actor->floorz - actor->dropoffz > FRACUNIT*24 &&
 	actor->z <= actor->floorz && !(actor->flags & (MF_DROPOFF|MF_FLOAT)) &&
@@ -853,7 +853,7 @@ static boolean P_LookForPlayers(mobj_t *actor, boolean allaround)
 
   c = 0;
 
-  stopc = demo_version < 203 && !demo_compatibility && monsters_remember ?
+  stopc = demo_version < DV_MBF && !demo_compatibility && monsters_remember ?
     MAXPLAYERS : 2;       // killough 9/9/98
 
   for (;; actor->lastlook = (actor->lastlook+1)&(MAXPLAYERS-1))
@@ -869,7 +869,7 @@ static boolean P_LookForPlayers(mobj_t *actor, boolean allaround)
         // There are no more desyncs on Donce's demos on horror.wad
 
         // Use last known enemy if no players sighted -- killough 2/15/98:
-        if (demo_version < 203 && !demo_compatibility && monsters_remember)
+        if (demo_version < DV_MBF && !demo_compatibility && monsters_remember)
         {
           if (actor->lastenemy && actor->lastenemy->health > 0)
           {
@@ -897,7 +897,7 @@ static boolean P_LookForPlayers(mobj_t *actor, boolean allaround)
 
       // killough 9/9/98: give monsters a threshold towards getting players
       // (we don't want it to be too easy for a player with dogs :)
-      if (demo_version >= 203 && !comp[comp_pursuit])
+      if (demo_version >= DV_MBF && !comp[comp_pursuit])
 	actor->threshold = 60;
 
       return true;
@@ -927,7 +927,7 @@ static boolean P_LookForMonsters(mobj_t *actor, boolean allaround)
       return true;
     }
 
-  if (demo_version < 203)  // Old demos do not support monster-seeking bots
+  if (demo_version < DV_MBF)  // Old demos do not support monster-seeking bots
     return false;
 
   // Search the threaded list corresponding to this object's potential targets
@@ -944,22 +944,33 @@ static boolean P_LookForMonsters(mobj_t *actor, boolean allaround)
       current_actor = actor;
       current_allaround = allaround;
 
+      // There is a bug in cl11+ that causes the player to get added
+      //   to the monster friend list when damaged to below 50% health.
+      // This causes all monsters to believe friend monsters exist.
+      // The search algorithm is expensive and massively so on maps with many monsters.
+      // We still need to match rng calls for demo sync, but PIT_FindTarget is a no op.
+      if (((mobj_t *) cap->cnext)->player && cap->cnext == cap->cprev)
+      {
+        P_Random(pr_friends);
+        return false;
+      }
+
       // Search first in the immediate vicinity.
 
-      if (!P_BlockThingsIterator(x, y, PIT_FindTarget))
+      if (!P_BlockThingsIterator(x, y, PIT_FindTarget, true))
 	return true;
 
       for (d=1; d<5; d++)
 	{
 	  int i = 1 - d;
 	  do
-	    if (!P_BlockThingsIterator(x+i, y-d, PIT_FindTarget) ||
-		!P_BlockThingsIterator(x+i, y+d, PIT_FindTarget))
+	    if (!P_BlockThingsIterator(x+i, y-d, PIT_FindTarget, true) ||
+		!P_BlockThingsIterator(x+i, y+d, PIT_FindTarget, true))
 	      return true;
 	  while (++i < d);
 	  do
-	    if (!P_BlockThingsIterator(x-d, y+i, PIT_FindTarget) ||
-		!P_BlockThingsIterator(x+d, y+i, PIT_FindTarget))
+	    if (!P_BlockThingsIterator(x-d, y+i, PIT_FindTarget, true) ||
+		!P_BlockThingsIterator(x+d, y+i, PIT_FindTarget, true))
 	      return true;
 	  while (--i + d >= 0);
 	}
@@ -1202,7 +1213,7 @@ void A_Chase(mobj_t *actor)
 
   if (!actor->threshold)
   {
-    if (demo_version < 203)
+    if (demo_version < DV_MBF)
       {   // killough 9/9/98: for backward demo compatibility
 	if (netgame && !P_CheckSight(actor, actor->target) &&
 	    P_LookForPlayers(actor, true))
@@ -1715,7 +1726,7 @@ static boolean P_HealCorpse(mobj_t* actor, int radius, statenum_t healstate, sfx
               // Call PIT_VileCheck to check
               // whether object is a corpse
               // that canbe raised.
-              if (!P_BlockThingsIterator(bx,by,PIT_VileCheck))
+              if (!P_BlockThingsIterator(bx, by, PIT_VileCheck, true))
                 {
 		  mobjinfo_t *info;
 
@@ -1758,7 +1769,7 @@ static boolean P_HealCorpse(mobj_t* actor, int radius, statenum_t healstate, sfx
                   corpsehit->health = info->spawnhealth;
 		  P_SetTarget(&corpsehit->target, NULL);  // killough 11/98
 
-		  if (demo_version >= 203)
+		  if (demo_version >= DV_MBF)
 		    {         // kilough 9/9/98
 		      P_SetTarget(&corpsehit->lastenemy, NULL);
 		      corpsehit->flags &= ~MF_JUSTHIT;
@@ -1852,7 +1863,7 @@ void A_VileTarget(mobj_t *actor)
 
   // killough 12/98: fix Vile fog coordinates
   fog = P_SpawnMobj(actor->target->x,
-                    demo_version < 203 ? actor->target->x : actor->target->y,
+                    demo_version < DV_MBF ? actor->target->x : actor->target->y,
                     actor->target->z,MT_FIRE);
 
   P_SetTarget(&actor->tracer, fog);   // killough 11/98
@@ -2021,7 +2032,7 @@ void A_BetaSkullAttack(mobj_t *actor)
 {
   int damage;
 
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
 
   if (!actor->target || actor->target->type == MT_SKULL)
@@ -2038,7 +2049,7 @@ void A_BetaSkullAttack(mobj_t *actor)
 
 void A_Stop(mobj_t *actor)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   actor->momx = actor->momy = actor->momz = 0;
 }
@@ -2186,13 +2197,13 @@ void A_Scream(mobj_t *actor)
 
 void A_XScream(mobj_t *actor)
 {
-  S_StartSound(actor, sfx_slop);
+  S_StartSoundEx(actor, sfx_slop);
 }
 
 void A_Pain(mobj_t *actor)
 {
   if (actor->info->painsound)
-    S_StartSound(actor, actor->info->painsound);
+    S_StartSoundPain(actor, actor->info->painsound);
 }
 
 void A_Fall(mobj_t *actor)
@@ -2204,7 +2215,7 @@ void A_Fall(mobj_t *actor)
 // killough 11/98: kill an object
 void A_Die(mobj_t *actor)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   P_DamageMobj(actor, NULL, NULL, actor->health);
 }
@@ -2224,7 +2235,7 @@ void A_Explode(mobj_t *thingy)
 
 void A_Detonate(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   P_RadiusAttack(mo, mo->target, mo->info->damage, mo->info->damage);
 }
@@ -2242,7 +2253,7 @@ void A_Mushroom(mobj_t *actor)
   fixed_t misc1 = actor->state->misc1 ? actor->state->misc1 : FRACUNIT*4;
   fixed_t misc2 = actor->state->misc2 ? actor->state->misc2 : FRACUNIT/2;
 
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   A_Explode(actor);               // make normal explosion
 
@@ -2698,7 +2709,7 @@ void A_PlayerScream(mobj_t *mo)
   int sound = sfx_pldeth;  // Default death sound.
   if (gamemode != shareware && mo->health < -50) // killough 12/98
     sound = sfx_pdiehi;   // IF THE PLAYER DIES LESS THAN -50% WITHOUT GIBBING
-  S_StartSound(mo, sound);
+  S_StartSoundEx(mo, sound);
 }
 
 //
@@ -2737,7 +2748,7 @@ void A_KeenDie(mobj_t* mo)
 
 void A_Spawn(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   if (mo->state->misc1)
     {
@@ -2754,21 +2765,21 @@ void A_Spawn(mobj_t *mo)
 
 void A_Turn(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   mo->angle += (angle_t)(((uint64_t) mo->state->misc1 << 32) / 360);
 }
 
 void A_Face(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   mo->angle = (angle_t)(((uint64_t) mo->state->misc1 << 32) / 360);
 }
 
 void A_Scratch(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   mo->target && (A_FaceTarget(mo), P_CheckMeleeRange(mo)) ?
     mo->state->misc2 ? S_StartSound(mo, mo->state->misc2) : (void) 0,
@@ -2777,14 +2788,14 @@ void A_Scratch(mobj_t *mo)
 
 void A_PlaySound(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
-  S_StartSound(mo->state->misc2 ? NULL : mo, mo->state->misc1);
+  S_StartSoundOrigin(mo, mo->state->misc2 ? NULL : mo, mo->state->misc1);
 }
 
 void A_RandomJump(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   if (P_Random(pr_randomjump) < mo->state->misc2)
     P_SetMobjState(mo, mo->state->misc1);
@@ -2796,7 +2807,7 @@ void A_RandomJump(mobj_t *mo)
 
 void A_LineEffect(mobj_t *mo)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
   if (!(mo->intflags & MIF_LINEDONE))                // Unless already used up
     {
@@ -3259,6 +3270,7 @@ void A_JumpIfFlagsSet(mobj_t* actor)
 void A_AddFlags(mobj_t* actor)
 {
   unsigned int flags, flags2;
+  boolean update_blockmap;
 
   if (!mbf21 || !actor)
     return;
@@ -3266,8 +3278,19 @@ void A_AddFlags(mobj_t* actor)
   flags  = actor->state->args[0];
   flags2 = actor->state->args[1];
 
+  // unlink/relink the thing from the blockmap if
+  // the NOBLOCKMAP or NOSECTOR flags are added
+  update_blockmap = ((flags & MF_NOBLOCKMAP) && !(actor->flags & MF_NOBLOCKMAP))
+                    || ((flags & MF_NOSECTOR) && !(actor->flags & MF_NOSECTOR));
+
+  if (update_blockmap)
+    P_UnsetThingPosition(actor);
+
   actor->flags  |= flags;
   actor->flags2 |= flags2;
+
+  if (update_blockmap)
+    P_SetThingPosition(actor);
 }
 
 //
@@ -3279,6 +3302,7 @@ void A_AddFlags(mobj_t* actor)
 void A_RemoveFlags(mobj_t* actor)
 {
   unsigned int flags, flags2;
+  boolean update_blockmap;
 
   if (!mbf21 || !actor)
     return;
@@ -3286,8 +3310,19 @@ void A_RemoveFlags(mobj_t* actor)
   flags  = actor->state->args[0];
   flags2 = actor->state->args[1];
 
+  // unlink/relink the thing from the blockmap if
+  // the NOBLOCKMAP or NOSECTOR flags are removed
+  update_blockmap = ((flags & MF_NOBLOCKMAP) && (actor->flags & MF_NOBLOCKMAP))
+                    || ((flags & MF_NOSECTOR) && (actor->flags & MF_NOSECTOR));
+
+  if (update_blockmap)
+    P_UnsetThingPosition(actor);
+
   actor->flags  &= ~flags;
   actor->flags2 &= ~flags2;
+
+  if (update_blockmap)
+    P_SetThingPosition(actor);
 }
 
 //----------------------------------------------------------------------------
