@@ -38,6 +38,7 @@
 #include "p_mobj.h"
 #include "p_setup.h"
 #include "p_spec.h"
+#include "p_user.h"
 #include "r_defs.h"
 #include "r_main.h"
 #include "r_state.h"
@@ -149,7 +150,7 @@ int P_GetFriction(const mobj_t *mo, int *frictionfactor)
   // friction value (muddy has precedence over icy).
 
   if (!(mo->flags & (MF_NOCLIP|MF_NOGRAVITY)) 
-      && (demo_version >= 203 || (mo->player && !compatibility)) &&
+      && (demo_version >= DV_MBF || (mo->player && !compatibility)) &&
       variable_friction)
     for (m = mo->touching_sectorlist; m; m = m->m_tnext)
       if ((sec = m->m_sector)->special & FRICTION_MASK &&
@@ -157,7 +158,7 @@ int P_GetFriction(const mobj_t *mo, int *frictionfactor)
 	  (mo->z <= sec->floorheight ||
 	   (sec->heightsec != -1 &&
 	    mo->z <= sectors[sec->heightsec].floorheight &&
-	    demo_version >= 203)))
+	    demo_version >= DV_MBF)))
 	friction = sec->friction, movefactor = sec->movefactor;
   
   if (frictionfactor)
@@ -178,7 +179,7 @@ int P_GetMoveFactor(const mobj_t *mo, int *frictionp)
 
   // Restore original Boom friction code for
   // demo compatibility
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
   {
     int momentum;
 
@@ -256,7 +257,7 @@ boolean P_TeleportMove(mobj_t *thing, fixed_t x, fixed_t y, boolean boss)
 
   // killough 8/9/98: make telefragging more consistent, preserve compatibility
   telefrag = thing->player || 
-    (comp[comp_telefrag] || demo_version < 203 ? gamemap==30 : boss);
+    (comp[comp_telefrag] || demo_version < DV_MBF ? gamemap==30 : boss);
 
   // kill anything occupying the position
 
@@ -294,7 +295,7 @@ boolean P_TeleportMove(mobj_t *thing, fixed_t x, fixed_t y, boolean boss)
 
   for (bx=xl ; bx<=xh ; bx++)
     for (by=yl ; by<=yh ; by++)
-      if (!P_BlockThingsIterator(bx,by,PIT_StompThing))
+      if (!P_BlockThingsIterator(bx, by, PIT_StompThing, true))
         return false;
 
   // the move is ok,
@@ -636,7 +637,7 @@ static boolean PIT_CheckThing(mobj_t *thing) // killough 3/26/98: make static
         P_DamageMobj(thing, tmthing, tmthing->target, damage);
 
         numspechit = 0;
-        return (true);
+        return true;
       }
 
       // damage / explode
@@ -780,7 +781,7 @@ boolean P_CheckPosition(mobj_t *thing, fixed_t x, fixed_t y)
   // Whether object can get out of a sticky situation:
   tmunstuck = thing->player &&          // only players
     thing->player->mo == thing &&       // not voodoo dolls
-    demo_version >= 203;                // not under old demos
+    demo_version >= DV_MBF;             // not under old demos
 
   // The base floor / ceiling is from the subsector
   // that contains the point.
@@ -808,7 +809,7 @@ boolean P_CheckPosition(mobj_t *thing, fixed_t x, fixed_t y)
 
   for (bx=xl ; bx<=xh ; bx++)
     for (by=yl ; by<=yh ; by++)
-      if (!P_BlockThingsIterator(bx,by,PIT_CheckThing))
+      if (!P_BlockThingsIterator(bx, by, PIT_CheckThing, !(tmthing->flags2 & MF2_RIP)))
         return false;
 
   // check lines
@@ -889,7 +890,7 @@ boolean P_TryMove(mobj_t *thing, fixed_t x, fixed_t y, boolean dropoff)
 			   (tmfloorz-tmdropoffz > 128*FRACUNIT || 
 			    !thing->target || thing->target->z >tmdropoffz)))
 	    {
-	      if (!monkeys || demo_version < 203 ?
+	      if (!monkeys || demo_version < DV_MBF ?
 		  tmfloorz - tmdropoffz > 24*FRACUNIT :
 		  thing->floorz  - tmfloorz > 24*FRACUNIT ||
 		  thing->dropoffz - tmdropoffz > 24*FRACUNIT)
@@ -930,6 +931,7 @@ boolean P_TryMove(mobj_t *thing, fixed_t x, fixed_t y, boolean dropoff)
   // killough 11/98: simplified
 
   if (!(thing->flags & (MF_TELEPORT | MF_NOCLIP)))
+  {
     while (numspechit--)
       if (spechit[numspechit]->special)  // see if the line was crossed
 	{
@@ -938,6 +940,10 @@ boolean P_TryMove(mobj_t *thing, fixed_t x, fixed_t y, boolean dropoff)
 	      P_PointOnLineSide(thing->x, thing->y, spechit[numspechit]))
 	    P_CrossSpecialLine(spechit[numspechit], oldside, thing, false);
 	}
+    // There are checks elsewhere for numspechit == 0, so we don't want to
+    // leave numspechit == -1.
+    numspechit = 0;
+  }
 
   return true;
 }
@@ -1151,7 +1157,7 @@ static void P_HitSlideLine(line_t *ld)
 
   // killough 10/98: only bounce if hit hard (prevents wobbling)
 
-  if (demo_version >= 203)
+  if (demo_version >= DV_MBF)
   {
   icyfloor = 
      P_AproxDistance(tmxmove, tmymove) > 4*FRACUNIT &&
@@ -1161,7 +1167,6 @@ static void P_HitSlideLine(line_t *ld)
   }
   else
   {
-    extern boolean onground;
     icyfloor = !compatibility &&
     variable_friction &&
     slidemo->player &&
@@ -1173,7 +1178,7 @@ static void P_HitSlideLine(line_t *ld)
     {
       if (icyfloor && abs(tmymove) > abs(tmxmove))
 	{
-	  S_StartSound(slidemo,sfx_oof); // oooff!
+	  S_StartSoundPreset(slidemo, sfx_oof, PITCH_FULL); // oooff!
 	  tmxmove /= 2; // absorb half the momentum
 	  tmymove = -tmymove/2;
 	}
@@ -1186,7 +1191,7 @@ static void P_HitSlideLine(line_t *ld)
     {
       if (icyfloor && abs(tmxmove) > abs(tmymove))
 	{
-	  S_StartSound(slidemo,sfx_oof); // oooff!
+	  S_StartSoundPreset(slidemo, sfx_oof, PITCH_FULL); // oooff!
 	  tmxmove = -tmxmove/2; // absorb half the momentum
 	  tmymove /= 2;
 	}
@@ -1218,7 +1223,7 @@ static void P_HitSlideLine(line_t *ld)
 
   if (icyfloor && deltaangle > ANG45 && deltaangle < ANG90+ANG45)
     {
-      S_StartSound(slidemo,sfx_oof); // oooff!
+      S_StartSoundPreset(slidemo, sfx_oof, PITCH_FULL); // oooff!
       moveangle = lineangle - deltaangle;
       movelen /= 2; // absorb
       moveangle >>= ANGLETOFINESHIFT;
@@ -1362,7 +1367,7 @@ void P_SlideMove(mobj_t *mo)
 	    if (!P_TryMove(mo, mo->x + mo->momx, mo->y, true))
 	      // [FG] Compatibility bug in P_SlideMove
 	      // http://prboom.sourceforge.net/mbf-bugs.html
-	      if (demo_version == 201)
+	      if (demo_version == DV_BOOM201)
 		mo->momx = mo->momy = 0;
 
 	  break;
@@ -1530,7 +1535,7 @@ static boolean PTR_AimTraverse (intercept_t *in)
 //
 static boolean PTR_ShootTraverse(intercept_t *in)
 {
-  fixed_t slope, dist, thingtopslope, thingbottomslope, x, y, z, frac;
+  fixed_t dist, thingtopslope, thingbottomslope, x, y, z, frac;
   mobj_t *th;
 
   if (in->isaline)
@@ -1551,15 +1556,15 @@ static boolean PTR_ShootTraverse(intercept_t *in)
 	  // backsector can be NULL when emulating missing back side.
 	  if (li->backsector == NULL)
 	  {
-	    if ((slope = FixedDiv(openbottom - shootz , dist)) <= aimslope &&
-	        (slope = FixedDiv(opentop - shootz , dist)) >= aimslope)
+	    if (FixedDiv(openbottom - shootz , dist) <= aimslope &&
+	        FixedDiv(opentop - shootz , dist) >= aimslope)
 	      return true;      // shot continues
 	  }
 	  else
 	  if ((li->frontsector->floorheight==li->backsector->floorheight ||
-	       (slope = FixedDiv(openbottom - shootz , dist)) <= aimslope) &&
+	       FixedDiv(openbottom - shootz , dist) <= aimslope) &&
 	      (li->frontsector->ceilingheight==li->backsector->ceilingheight ||
-	       (slope = FixedDiv (opentop - shootz , dist)) >= aimslope))
+	       FixedDiv (opentop - shootz , dist) >= aimslope))
 	    return true;      // shot continues
 	}
 
@@ -2001,7 +2006,7 @@ void P_RadiusAttack(mobj_t *spot, mobj_t *source, int damage, int distance)
 
   for (y=yl ; y<=yh ; y++)
     for (x=xl ; x<=xh ; x++)
-      P_BlockThingsIterator(x, y, PIT_RadiusAttack);
+      P_BlockThingsIterator(x, y, PIT_RadiusAttack, false);
 }
 
 //
@@ -2115,7 +2120,7 @@ boolean P_ChangeSector(sector_t *sector,boolean crunch)
 
   for (x=sector->blockbox[BOXLEFT] ; x<= sector->blockbox[BOXRIGHT] ; x++)
     for (y=sector->blockbox[BOXBOTTOM];y<= sector->blockbox[BOXTOP] ; y++)
-      P_BlockThingsIterator (x, y, PIT_ChangeSector);
+      P_BlockThingsIterator (x, y, PIT_ChangeSector, false);
 
   return nofit;
 }
@@ -2132,7 +2137,7 @@ boolean P_CheckSector(sector_t *sector,boolean crunch)
   msecnode_t *n;
 
   // killough 10/98: sometimes use Doom's method
-  if (comp[comp_floors] && (demo_version >= 203 || demo_compatibility))
+  if (comp[comp_floors] && (demo_version >= DV_MBF || demo_compatibility))
     return P_ChangeSector(sector,crunch);
 
   nofit = false;
@@ -2178,7 +2183,7 @@ msecnode_t *headsecnode = NULL;
 
 static msecnode_t *P_GetSecnode(void)
 {
-  msecnode_t *node = headsecnode;
+  msecnode_t *node;
 
   return headsecnode ?
     node = headsecnode, headsecnode = node->m_snext, node :
