@@ -22,6 +22,7 @@
 #include "d_items.h"
 #include "d_player.h"
 #include "doomstat.h"
+#include "g_nextweapon.h"
 #include "i_printf.h"
 #include "i_video.h" // uncapped
 #include "m_random.h"
@@ -32,6 +33,7 @@
 #include "p_mobj.h"
 #include "p_pspr.h"
 #include "p_tick.h"
+#include "p_user.h"
 #include "r_main.h"
 #include "s_sound.h"
 #include "sounds.h"
@@ -44,8 +46,6 @@
 
 #define BFGCELLS bfgcells        /* Ty 03/09/98 externalized in p_inter.c */
 
-extern void P_Thrust(player_t *, angle_t, fixed_t);
-
 // The following array holds the recoil values         // phares
 static struct
 {
@@ -80,8 +80,6 @@ void A_Recoil(player_t* player)
 
 static void P_SetPsprite(player_t *player, int position, statenum_t stnum)
 {
-  P_SetPspritePtr(player, &player->psprites[position], stnum);
-
   if (position == ps_weapon)
   {
     const weaponinfo_t wp = weaponinfo[player->readyweapon];
@@ -91,6 +89,8 @@ static void P_SetPsprite(player_t *player, int position, statenum_t stnum)
     else if (stnum == wp.downstate)
       player->switching = weapswitch_lowering;
   }
+
+  P_SetPspritePtr(player, &player->psprites[position], stnum);
 }
 
 //
@@ -156,11 +156,10 @@ static void P_BringUpWeapon(player_t *player)
     player->pendingweapon = player->readyweapon;
 
   if (player->pendingweapon == wp_chainsaw)
-    S_StartSound(player->mo, sfx_sawup);
+    S_StartSoundPitchEx(player->mo, sfx_sawup, PITCH_HALF);
 
   if (player->pendingweapon >= NUMWEAPONS)
   {
-    player->pendingweapon = NUMWEAPONS;
     I_Printf(VB_WARNING, "P_BringUpWeapon: weaponinfo overrun has occurred.");
   }
 
@@ -169,7 +168,7 @@ static void P_BringUpWeapon(player_t *player)
   player->pendingweapon = wp_nochange;
 
   // killough 12/98: prevent pistol from starting visibly at bottom of screen:
-  player->psprites[ps_weapon].sy = demo_version >= 203 ? 
+  player->psprites[ps_weapon].sy = demo_version >= DV_MBF ? 
     WEAPONBOTTOM+FRACUNIT*2 : WEAPONBOTTOM;
 
   P_SetPsprite(player, ps_weapon, newstate);
@@ -238,7 +237,7 @@ static int P_SwitchWeaponMBF21(player_t *player)
         checkweapon = wp_chainsaw;
         break;
       case 9:
-        if (have_ssg)
+        if (ALLOW_SSG)
           checkweapon = wp_supershotgun;
         break;
     }
@@ -268,6 +267,8 @@ int P_SwitchWeapon(player_t *player)
   int newweapon = currentweapon;
   int i = NUMWEAPONS+1;   // killough 5/2/98
 
+  G_NextWeaponReset();
+
   // [XA] use fixed behavior for mbf21. no need
   // for a discrete compat option for this, as
   // it doesn't impact demo playback (weapon
@@ -317,7 +318,7 @@ int P_SwitchWeapon(player_t *player)
           newweapon = wp_chainsaw;
         break;
       case 9:
-        if (player->weaponowned[wp_supershotgun] && have_ssg &&
+        if (player->weaponowned[wp_supershotgun] && ALLOW_SSG &&
             player->ammo[am_shell] >= (demo_compatibility ? 3 : 2))
           newweapon = wp_supershotgun;
         break;
@@ -485,7 +486,7 @@ void A_WeaponReady(player_t *player, pspdef_t *psp)
     P_SetMobjState(player->mo, S_PLAY);
 
   if (player->readyweapon == wp_chainsaw && psp->state == &states[S_SAW])
-    S_StartSound(player->mo, sfx_sawidl);
+    S_StartSoundPitch(player->mo, sfx_sawidl, PITCH_HALF);
 
   // check for change
   //  if player is dead, put the weapon away
@@ -588,7 +589,11 @@ void A_Lower(player_t *player, pspdef_t *psp)
       return;
     }
 
-  player->readyweapon = player->pendingweapon;
+  if (player->pendingweapon < NUMWEAPONS || !mbf21)
+  {
+    player->lastweapon = player->readyweapon;
+    player->readyweapon = player->pendingweapon;
+  }
 
   P_BringUpWeapon(player);
 }
@@ -630,7 +635,7 @@ static void A_FireSomething(player_t* player,int adder)
 
   // killough 3/27/98: prevent recoil in no-clipping mode
   if (!(player->mo->flags & MF_NOCLIP))
-    if (weapon_recoil && (demo_version >= 203 || !compatibility))
+    if (weapon_recoil && (demo_version >= DV_MBF || !compatibility))
       P_Thrust(player, ANG180 + player->mo->angle,
                2048*recoil_values[player->readyweapon].thrust);          // phares
 }
@@ -650,6 +655,18 @@ void A_GunFlash(player_t *player, pspdef_t *psp)
 // WEAPON ATTACKS
 //
 
+static angle_t saved_angle;
+
+static void SavePlayerAngle(player_t *player)
+{
+  saved_angle = player->mo->angle;
+}
+
+static void AddToTicAngle(player_t *player)
+{
+  player->ticangle += player->mo->angle - saved_angle;
+}
+
 //
 // A_Punch
 //
@@ -672,7 +689,7 @@ void A_Punch(player_t *player, pspdef_t *psp)
   range = (mbf21 ? player->mo->info->meleerange : MELEERANGE);
 
   // killough 8/2/98: make autoaiming prefer enemies
-  if (demo_version<203 ||
+  if (demo_version < DV_MBF ||
       (slope = P_AimLineAttack(player->mo, angle, range, MF_FRIEND),
        !linetarget))
     slope = P_AimLineAttack(player->mo, angle, range, 0);
@@ -682,12 +699,14 @@ void A_Punch(player_t *player, pspdef_t *psp)
   if (!linetarget)
     return;
 
-  S_StartSound(player->mo, sfx_punch);
+  S_StartSoundEx(player->mo, sfx_punch);
 
   // turn to face target
 
+  SavePlayerAngle(player);
   player->mo->angle = R_PointToAngle2(player->mo->x, player->mo->y,
                                       linetarget->x, linetarget->y);
+  AddToTicAngle(player);
 }
 
 //
@@ -709,7 +728,7 @@ void A_Saw(player_t *player, pspdef_t *psp)
   range = (mbf21 ? player->mo->info->meleerange : MELEERANGE) + 1;
 
   // killough 8/2/98: make autoaiming prefer enemies
-  if (demo_version<203 ||
+  if (demo_version < DV_MBF ||
       (slope = P_AimLineAttack(player->mo, angle, range, MF_FRIEND),
        !linetarget))
     slope = P_AimLineAttack(player->mo, angle, range, 0);
@@ -720,16 +739,17 @@ void A_Saw(player_t *player, pspdef_t *psp)
 
   if (!linetarget)
     {
-      S_StartSound(player->mo, sfx_sawful);
+      S_StartSoundPitchEx(player->mo, sfx_sawful, PITCH_HALF);
       return;
     }
 
-  S_StartSound(player->mo, sfx_sawhit);
+  S_StartSoundPitchEx(player->mo, sfx_sawhit, PITCH_HALF);
 
   // turn to face target
   angle = R_PointToAngle2(player->mo->x, player->mo->y,
                           linetarget->x, linetarget->y);
 
+  SavePlayerAngle(player);
   if (angle - player->mo->angle > ANG180)
     if ((signed int) (angle - player->mo->angle) < -ANG90/20)
       player->mo->angle = angle + ANG90/21;
@@ -740,6 +760,7 @@ void A_Saw(player_t *player, pspdef_t *psp)
       player->mo->angle = angle - ANG90/21;
     else
       player->mo->angle += ANG90/20;
+  AddToTicAngle(player);
 
   player->mo->flags |= MF_JUSTATTACKED;
 }
@@ -796,7 +817,6 @@ void A_FireOldBFG(player_t *player, pspdef_t *psp)
       angle_t an = mo->angle;
       angle_t an1 = ((P_Random(pr_bfg)&127) - 64) * (ANG90/768) + an;
       angle_t an2 = ((P_Random(pr_bfg)&127) - 64) * (ANG90/640) + ANG90;
-      extern int autoaim;
 
       if (autoaim || !beta_emulation)
 	{
@@ -871,7 +891,7 @@ static void P_BulletSlope(mobj_t *mo)
   angle_t an = mo->angle;    // see which target is to be aimed at
 
   // killough 8/2/98: make autoaiming prefer enemies
-  int mask = demo_version < 203 ? 0 : MF_FRIEND;
+  int mask = demo_version < DV_MBF ? 0 : MF_FRIEND;
 
   if (direct_vertical_aiming)
   {
@@ -913,7 +933,7 @@ void P_GunShot(mobj_t *mo, boolean accurate)
 
 void A_FirePistol(player_t *player, pspdef_t *psp)
 {
-  S_StartSound(player->mo, sfx_pistol);
+  S_StartSoundPistol(player->mo, sfx_pistol);
 
   P_SetMobjState(player->mo, S_PLAY_ATK2);
   P_SubtractAmmo(player, 1);
@@ -932,7 +952,7 @@ void A_FireShotgun(player_t *player, pspdef_t *psp)
 {
   int i;
 
-  S_StartSound(player->mo, sfx_shotgn);
+  S_StartSoundShotgun(player->mo, sfx_shotgn);
   P_SetMobjState(player->mo, S_PLAY_ATK2);
 
   P_SubtractAmmo(player, 1);
@@ -954,7 +974,7 @@ void A_FireShotgun2(player_t *player, pspdef_t *psp)
 {
   int i;
 
-  S_StartSound(player->mo, sfx_dshtgn);
+  S_StartSoundSSG(player->mo, sfx_dshtgn);
   P_SetMobjState(player->mo, S_PLAY_ATK2);
   P_SubtractAmmo(player, 2);
 
@@ -982,7 +1002,7 @@ void A_FireShotgun2(player_t *player, pspdef_t *psp)
 
 void A_FireCGun(player_t *player, pspdef_t *psp)
 {
-  S_StartSound(player->mo, sfx_pistol);
+  S_StartSoundCGun(player->mo, sfx_pistol);
 
   if (!player->ammo[weaponinfo[player->readyweapon].ammo])
     return;
@@ -1039,7 +1059,7 @@ void A_BFGSpray(mobj_t *mo)
       // mo->target is the originator (player) of the missile
 
       // killough 8/2/98: make autoaiming prefer enemies
-      if (demo_version < 203 || 
+      if (demo_version < DV_MBF || 
 	  (P_AimLineAttack(mo->target, an, 16*64*FRACUNIT, MF_FRIEND), 
 	   !linetarget))
 	P_AimLineAttack(mo->target, an, 16*64*FRACUNIT, 0);
@@ -1063,7 +1083,7 @@ void A_BFGSpray(mobj_t *mo)
 
 void A_BFGsound(player_t *player, pspdef_t *psp)
 {
-  S_StartSound(player->mo, sfx_bfg);
+  S_StartSoundBFG(player->mo, sfx_bfg);
 }
 
 //
@@ -1301,10 +1321,12 @@ void A_WeaponMeleeAttack(player_t *player, pspdef_t *psp)
     return;
 
   // un-missed!
-  S_StartSound(player->mo, hitsound);
+  S_StartSoundEx(player->mo, hitsound);
 
   // turn to face target
+  SavePlayerAngle(player);
   player->mo->angle = R_PointToAngle2(player->mo->x, player->mo->y, linetarget->x, linetarget->y);
+  AddToTicAngle(player);
 }
 
 //
@@ -1318,7 +1340,8 @@ void A_WeaponSound(player_t *player, pspdef_t *psp)
   if (!mbf21 || !psp->state)
     return;
 
-  S_StartSound(psp->state->args[1] ? NULL : player->mo, psp->state->args[0]);
+  S_StartSoundOrigin(player->mo, (psp->state->args[1] ? NULL : player->mo),
+                     psp->state->args[0]);
 }
 
 //
