@@ -28,6 +28,7 @@
 #include "doomstat.h"
 #include "i_system.h"
 #include "info.h"
+#include "m_array.h"
 #include "m_random.h"
 #include "p_enemy.h"
 #include "p_maputl.h"
@@ -36,6 +37,7 @@
 #include "p_saveg.h"
 #include "p_spec.h"
 #include "p_tick.h"
+#include "r_data.h"
 #include "r_defs.h"
 #include "r_state.h"
 #include "w_wad.h" // [FG] W_LumpLength()
@@ -972,6 +974,33 @@ static void saveg_read_player_t(player_t *str)
         str->slope = 0;
         str->maxkilldiscount = 0;
     }
+
+    if (saveg_compat > saveg_woof1300)
+    {
+        // [Woof!]: int num_visitedlevels;
+        str->num_visitedlevels = saveg_read32();
+
+        // [Woof!]: level_t *visitedlevels;
+        array_clear(str->visitedlevels);
+        for (int i = 0; i < str->num_visitedlevels; ++i)
+        {
+            level_t level = {0};
+            level.episode = saveg_read32();
+            level.map = saveg_read32();
+            array_push(str->visitedlevels, level);
+        }
+
+        // [Woof!]: weapontype_t lastweapon;
+        str->lastweapon = saveg_read_enum();
+    }
+    else
+    {
+        str->num_visitedlevels = 0;
+        array_clear(str->visitedlevels);
+        str->lastweapon = wp_nochange;
+    }
+
+    str->nextweapon = str->readyweapon;
 }
 
 static void saveg_write_player_t(player_t *str)
@@ -1124,6 +1153,20 @@ static void saveg_write_player_t(player_t *str)
 
     // [Woof!]: int maxkilldiscount;
     saveg_write32(str->maxkilldiscount);
+
+    // [Woof!]: int num_visitedlevels;
+    saveg_write32(str->num_visitedlevels);
+
+    // [Woof!]: level_t *visitedlevels;
+    level_t *level;
+    array_foreach(level, str->visitedlevels)
+    {
+        saveg_write32(level->episode);
+        saveg_write32(level->map);
+    }
+
+    // [Woof!]: weapontype_t lastweapon;
+    saveg_write_enum(str->lastweapon);
 }
 
 
@@ -2085,7 +2128,6 @@ void P_UnArchiveWorld (void)
     {
       // [crispy] add overflow guard for the flattranslation[] array
       short floorpic, ceilingpic;
-      extern int numflats;
 
       // killough 10/98: load full floor & ceiling heights, including fractions
 
