@@ -810,6 +810,8 @@ boolean P_CanUnlockGenDoor(struct line_s *line, struct player_s *player);
 
 int P_SectorActive(special_e t, struct sector_s *s);
 
+boolean P_IsDeathExit(struct sector_s *sec);
+
 boolean P_IsSecret(struct sector_s *sec);
 
 boolean P_WasSecret(struct sector_s *sec);
@@ -964,6 +966,8 @@ void P_ShootSpecialLine(struct mobj_s *thing, struct line_s *line);
 void P_CrossSpecialLine(struct line_s *, int side, struct mobj_s *thing,
                         boolean bossaction); // killough 11/98
 
+extern int disable_nuke;  // killough 12/98: nukage disabling cheat
+
 void P_PlayerInSpecialSector(struct player_s *player);
 
 // p_lights
