@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "d_main.h"
 #include "d_think.h"
 #include "doomdef.h"
 #include "doomstat.h"
@@ -43,6 +44,7 @@
 #include "r_main.h"
 #include "r_sky.h"
 #include "r_state.h"
+#include "v_fmt.h"
 #include "v_video.h" // cr_dark, cr_shaded
 #include "w_wad.h"
 #include "z_zone.h"
@@ -231,7 +233,7 @@ static void R_GenerateComposite(int texnum)
 
   for (; --i >=0; patch++)
     {
-      patch_t *realpatch = W_CacheLumpNum(patch->patch, PU_CACHE);
+      patch_t *realpatch = V_CachePatchNum(patch->patch, PU_CACHE);
       int x, x1 = patch->originx, x2 = x1 + SHORT(realpatch->width);
       const int *cofs = realpatch->columnofs - x1;
 
@@ -347,7 +349,7 @@ static void R_GenerateLookup(int texnum, int *const errors)
   while (--i >= 0)
     {
       int pat = patch->patch;
-      const patch_t *realpatch = W_CacheLumpNum(pat, PU_CACHE);
+      const patch_t *realpatch = V_CachePatchNum(pat, PU_CACHE);
       int x, x1 = patch++->originx, x2 = x1 + SHORT(realpatch->width);
       const int *cofs = realpatch->columnofs - x1;
       
@@ -385,7 +387,7 @@ static void R_GenerateLookup(int texnum, int *const errors)
       for (i = texture->patchcount, patch = texture->patches; --i >= 0;)
 	{
 	  int pat = patch->patch;
-	  const patch_t *realpatch = W_CacheLumpNum(pat, PU_CACHE);
+	  const patch_t *realpatch = V_CachePatchNum(pat, PU_CACHE);
 	  int x, x1 = patch++->originx, x2 = x1 + SHORT(realpatch->width);
 	  const int *cofs = realpatch->columnofs - x1;
 	  
@@ -832,7 +834,7 @@ void R_InitSpriteLumps(void)
       if (!(i&127))            // killough
         I_PutChar(VB_INFO, '.');
 
-      patch = W_CacheLumpNum(firstspritelump+i, PU_CACHE);
+      patch = V_CachePatchNum(firstspritelump+i, PU_CACHE);
       spritewidth[i] = SHORT(patch->width)<<FRACBITS;
       spriteoffset[i] = SHORT(patch->leftoffset)<<FRACBITS;
       spritetopoffset[i] = SHORT(patch->topoffset)<<FRACBITS;
@@ -848,6 +850,7 @@ void R_InitSpriteLumps(void)
 // killough 4/4/98: Add support for C_START/C_END markers
 //
 
+invul_mode_t invul_mode;
 static byte invul_orig[256];
 
 void R_InvulMode(void)
@@ -935,8 +938,7 @@ void R_InitTranMap(int progress)
   else
     {   // Compose a default transparent filter map based on PLAYPAL.
       unsigned char *playpal = W_CacheLumpName("PLAYPAL", PU_STATIC);
-      extern const char *D_DoomPrefDir(void);
-      char *fname = M_StringJoin(D_DoomPrefDir(), DIR_SEPARATOR_S, "tranmap.dat", NULL);
+      char *fname = M_StringJoin(D_DoomPrefDir(), DIR_SEPARATOR_S, "tranmap.dat");
       struct {
         unsigned char pct;
         unsigned char playpal[256*3]; // [FG] a palette has 256 colors saved as byte triples
@@ -1042,6 +1044,25 @@ void R_InitTranMap(int progress)
       Z_ChangeTag(playpal, PU_CACHE);
       free(fname);
     }
+
+  //!
+  // @category mod
+  // @arg <name>
+  //
+  // Dump tranmap lump.
+  //
+
+  int p = M_CheckParmWithArgs("-dumptranmap", 1);
+  if (p > 0)
+  {
+      char *path = malloc(strlen(myargv[p + 1]) + 5);
+      strcpy(path, myargv[p + 1]);
+      AddDefaultExtension(path, ".lmp");
+
+      M_WriteFile(path, main_tranmap, 256 * 256);
+
+      free(path);
+  }
 }
 
 //
@@ -1159,7 +1180,7 @@ void R_PrecacheLevel(void)
 
   for (i = numflats; --i >= 0; )
     if (hitlist[i])
-      W_CacheLumpNum(firstflat + i, PU_CACHE);
+      V_CacheFlatNum(firstflat + i, PU_CACHE);
 
   // Precache textures.
 
@@ -1185,7 +1206,7 @@ void R_PrecacheLevel(void)
         texture_t *texture = textures[i];
         int j = texture->patchcount;
         while (--j >= 0)
-          W_CacheLumpNum(texture->patches[j].patch, PU_CACHE);
+          V_CachePatchNum(texture->patches[j].patch, PU_CACHE);
       }
 
   // Precache sprites.
@@ -1207,7 +1228,7 @@ void R_PrecacheLevel(void)
             short *sflump = sprites[i].spriteframes[j].lump;
             int k = 7;
             do
-              W_CacheLumpNum(firstspritelump + sflump[k], PU_CACHE);
+              V_CachePatchNum(firstspritelump + sflump[k], PU_CACHE);
             while (--k >= 0);
           }
       }
@@ -1228,22 +1249,19 @@ boolean R_IsPatchLump (const int lump)
   if (lump < 0)
     return false;
 
-  size = W_LumpLength(lump);
+  patch = V_CachePatchNum(lump, PU_CACHE);
+
+  size = V_LumpSize(lump);
 
   // minimum length of a valid Doom patch
   if (size < 13)
     return false;
 
-  patch = (const patch_t *)W_CacheLumpNum(lump, PU_CACHE);
-
-  // [FG] detect patches in PNG format early
-  if (!memcmp(patch, "\211PNG\r\n\032\n", 8))
-    return false;
-
   width = SHORT(patch->width);
   height = SHORT(patch->height);
 
-  result = (height > 0 && height <= 16384 && width > 0 && width <= 16384 && width < size / 4);
+  result = (height > 0 && height <= 16384 && width > 0 && width <= 16384
+            && width < size / 4);
 
   if (result)
   {
