@@ -25,13 +25,16 @@
 #include "doomdef.h"
 #include "doomstat.h"
 #include "i_printf.h"
+#include "i_rumble.h"
 #include "i_sound.h"
 #include "i_system.h"
+#include "m_config.h"
 #include "m_misc.h"
 #include "m_random.h"
 #include "p_mobj.h"
 #include "s_musinfo.h" // [crispy] struct musinfo
 #include "s_sound.h"
+#include "s_trakinfo.h"
 #include "sounds.h"
 #include "u_mapinfo.h"
 #include "w_wad.h"
@@ -44,7 +47,6 @@ typedef struct channel_s
     sfxinfo_t *sfxinfo;   // sound information (if null, channel avail.)
     const mobj_t *origin; // origin of sound
     int volume;           // volume scale value for effect -- haleyjd 05/29/06
-    int pitch;            // pitch modifier -- haleyjd 06/03/06
     int handle;           // handle of the sound being played
     int o_priority;       // haleyjd 09/27/06: stored priority value
     int priority;         // current priority value
@@ -74,8 +76,7 @@ static musicinfo_t *mus_playing;
 // following is set
 //  by the defaults code in M_misc:
 // number of channels available
-int numChannels;
-int default_numChannels; // killough 9/98
+int snd_channels;
 
 // jff 3/17/98 to keep track of last IDMUS specified music num
 int idmusnum;
@@ -92,7 +93,7 @@ int idmusnum;
 static void S_StopChannel(int cnum)
 {
 #ifdef RANGECHECK
-    if (cnum < 0 || cnum >= numChannels)
+    if (cnum < 0 || cnum >= snd_channels)
     {
         I_Error("S_StopChannel: handle %d out of range\n", cnum);
     }
@@ -107,6 +108,17 @@ static void S_StopChannel(int cnum)
     }
 }
 
+void S_StopChannels(void)
+{
+    for (int i = 0; i < MAX_CHANNELS; i++)
+    {
+        I_StopSound(channels[i].handle);
+    }
+
+    memset(channels, 0, sizeof(channels));
+    memset(sobjs, 0, sizeof(sobjs));
+}
+
 //
 // S_AdjustSoundParams
 //
@@ -117,8 +129,7 @@ static void S_StopChannel(int cnum)
 // haleyjd: added priority scaling
 //
 static int S_AdjustSoundParams(const mobj_t *listener, const mobj_t *source,
-                               int chanvol, int *vol, int *sep, int *pitch,
-                               int *pri)
+                               int chanvol, int *vol, int *sep, int *pri)
 {
     return I_AdjustSoundParams(listener, source, chanvol, vol, sep, pri);
 }
@@ -145,7 +156,7 @@ static int S_getChannel(const mobj_t *origin, sfxinfo_t *sfxinfo, int priority,
     // kill old sound
     // killough 12/98: replace is_pickup hack with singularity flag
     // haleyjd 06/12/08: only if subchannel matches
-    for (cnum = 0; cnum < numChannels; cnum++)
+    for (cnum = 0; cnum < snd_channels; cnum++)
     {
         if (channels[cnum].sfxinfo && channels[cnum].singularity == singularity
             && channels[cnum].origin == origin)
@@ -156,13 +167,13 @@ static int S_getChannel(const mobj_t *origin, sfxinfo_t *sfxinfo, int priority,
     }
 
     // Find an open channel
-    if (cnum == numChannels)
+    if (cnum == snd_channels)
     {
         // haleyjd 09/28/06: it isn't necessary to look for playing sounds in
         // the same singularity class again, as we just did that above. Here
         // we are looking for an open channel. We will also keep track of the
         // channel found with the lowest sound priority while doing this.
-        for (cnum = 0; cnum < numChannels && channels[cnum].sfxinfo; cnum++)
+        for (cnum = 0; cnum < snd_channels && channels[cnum].sfxinfo; cnum++)
         {
             if (channels[cnum].priority > lowestpriority)
             {
@@ -173,7 +184,7 @@ static int S_getChannel(const mobj_t *origin, sfxinfo_t *sfxinfo, int priority,
     }
 
     // None available?
-    if (cnum == numChannels)
+    if (cnum == snd_channels)
     {
         // Look for lower priority
         // haleyjd: we have stored the channel found with the lowest priority
@@ -190,7 +201,7 @@ static int S_getChannel(const mobj_t *origin, sfxinfo_t *sfxinfo, int priority,
     }
 
 #ifdef RANGECHECK
-    if (cnum >= numChannels)
+    if (cnum >= snd_channels)
     {
         I_Error("S_getChannel: handle %d out of range\n", cnum);
     }
@@ -199,7 +210,8 @@ static int S_getChannel(const mobj_t *origin, sfxinfo_t *sfxinfo, int priority,
     return cnum;
 }
 
-void S_StartSound(const mobj_t *origin, int sfx_id)
+static void StartSound(const mobj_t *origin, int sfx_id,
+                       pitchrange_t pitch_range, rumble_type_t rumble_type)
 {
     int sep, pitch, o_priority, priority, singularity, cnum, handle;
     int volumeScale = 127;
@@ -229,25 +241,7 @@ void S_StartSound(const mobj_t *origin, int sfx_id)
     sfx = &S_sfx[sfx_id];
 
     // Initialize sound parameters
-    if (sfx->link)
-    {
-        pitch = sfx->pitch;
-        volumeScale += sfx->volume;
-    }
-    else
-    {
-        pitch = NORM_PITCH;
-    }
-
-    // haleyjd 09/29/06: rangecheck volumeScale now!
-    if (volumeScale < 0)
-    {
-        volumeScale = 0;
-    }
-    else if (volumeScale > 127)
-    {
-        volumeScale = 127;
-    }
+    pitch = NORM_PITCH;
 
     // haleyjd: modified so that priority value is always used
     // haleyjd: also modified to get and store proper singularity value
@@ -258,7 +252,7 @@ void S_StartSound(const mobj_t *origin, int sfx_id)
     // killough 3/7/98, 4/25/98: code rearranged slightly
 
     if (!S_AdjustSoundParams(players[displayplayer].mo, origin, volumeScale,
-                             &volume, &sep, &pitch, &priority))
+                             &volume, &sep, &priority))
     {
         return;
     }
@@ -266,11 +260,11 @@ void S_StartSound(const mobj_t *origin, int sfx_id)
     if (pitched_sounds)
     {
         // hacks to vary the sfx pitches
-        if (sfx_id >= sfx_sawup && sfx_id <= sfx_sawhit)
+        if (pitch_range == PITCH_HALF)
         {
             pitch += 8 - (M_Random() & 15);
         }
-        else if (sfx_id != sfx_itemup && sfx_id != sfx_tink)
+        else if (pitch_range == PITCH_FULL)
         {
             pitch += 16 - (M_Random() & 31);
         }
@@ -293,7 +287,7 @@ void S_StartSound(const mobj_t *origin, int sfx_id)
     }
 
 #ifdef RANGECHECK
-    if (cnum < 0 || cnum >= numChannels)
+    if (cnum < 0 || cnum >= snd_channels)
     {
         I_Error("S_StartSfxInfo: handle %d out of range\n", cnum);
     }
@@ -316,14 +310,18 @@ void S_StartSound(const mobj_t *origin, int sfx_id)
         channels[cnum].handle = handle;
 
         // haleyjd 05/29/06: record volume scale value
-        // haleyjd 06/03/06: record pitch too (wtf is going on here??)
         // haleyjd 09/27/06: store priority and singularity values (!!!)
         channels[cnum].volume = volumeScale;
-        channels[cnum].pitch = pitch;
         channels[cnum].o_priority = o_priority; // original priority
         channels[cnum].priority = priority;     // scaled priority
         channels[cnum].singularity = singularity;
         channels[cnum].idnum = I_SoundID(handle); // unique instance id
+
+        if (rumble_type != RUMBLE_NONE)
+        {
+            I_StartRumble(players[displayplayer].mo, origin, sfx, handle,
+                          rumble_type);
+        }
     }
     else // haleyjd: the sound didn't start, so clear the channel info
     {
@@ -331,6 +329,128 @@ void S_StartSound(const mobj_t *origin, int sfx_id)
     }
 }
 
+void S_StartSoundPitch(const mobj_t *origin, int sfx_id,
+                       pitchrange_t pitch_range)
+{
+    StartSound(origin, sfx_id, pitch_range, RUMBLE_NONE);
+}
+
+static boolean IsRumblePlayer(const mobj_t *mo)
+{
+    return (I_RumbleEnabled() && mo && mo == players[displayplayer].mo);
+}
+
+static rumble_type_t RumbleType(const mobj_t *mo, rumble_type_t rumble_type)
+{
+    return (IsRumblePlayer(mo) ? rumble_type : RUMBLE_NONE);
+}
+
+void S_StartSoundPitchEx(const mobj_t *origin, int sfx_id,
+                         pitchrange_t pitch_range)
+{
+    StartSound(origin, sfx_id, pitch_range, RumbleType(origin, RUMBLE_PLAYER));
+}
+
+void S_StartSoundPistol(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_PISTOL));
+}
+
+void S_StartSoundShotgun(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_SHOTGUN));
+}
+
+void S_StartSoundSSG(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_SSG));
+}
+
+void S_StartSoundCGun(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_CGUN));
+}
+
+void S_StartSoundBFG(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_BFG));
+}
+
+static rumble_type_t RumbleTypePreset(const mobj_t *origin, int sfx_id)
+{
+    if (IsRumblePlayer(origin))
+    {
+        switch (sfx_id)
+        {
+            case sfx_itemup:
+                return RUMBLE_ITEMUP;
+            case sfx_wpnup:
+                return RUMBLE_WPNUP;
+            case sfx_getpow:
+                return RUMBLE_GETPOW;
+            case sfx_oof:
+                return RUMBLE_OOF;
+        }
+    }
+    return RUMBLE_NONE;
+}
+
+void S_StartSoundPreset(const mobj_t *origin, int sfx_id,
+                        pitchrange_t pitch_range)
+{
+    StartSound(origin, sfx_id, pitch_range, RumbleTypePreset(origin, sfx_id));
+}
+
+void S_StartSoundPain(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_PAIN));
+}
+
+void S_StartSoundHitFloor(const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(origin, RUMBLE_HITFLOOR));
+}
+
+void S_StartSoundSource(const mobj_t *source, const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(source, RUMBLE_PLAYER));
+}
+
+static rumble_type_t RumbleTypeMissile(const mobj_t *source,
+                                       const mobj_t *origin)
+{
+    if (IsRumblePlayer(source))
+    {
+        if (origin)
+        {
+            switch (origin->type)
+            {
+                case MT_ROCKET:
+                    return RUMBLE_ROCKET;
+                case MT_PLASMA:
+                case MT_PLASMA1:
+                case MT_PLASMA2:
+                    return RUMBLE_PLASMA;
+
+                default:
+                    break;
+            }
+        }
+        return RUMBLE_PLAYER;
+    }
+    return RUMBLE_NONE;
+}
+
+void S_StartSoundMissile(const mobj_t *source, const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleTypeMissile(source, origin));
+}
+
+void S_StartSoundOrigin(const mobj_t *source, const mobj_t *origin, int sfx_id)
+{
+    StartSound(origin, sfx_id, PITCH_FULL, RumbleType(source, RUMBLE_ORIGIN));
+}
+
 //
 // S_StopSound
 //
@@ -344,7 +464,7 @@ void S_StopSound(const mobj_t *origin)
         return;
     }
 
-    for (cnum = 0; cnum < numChannels; ++cnum)
+    for (cnum = 0; cnum < snd_channels; ++cnum)
     {
         if (channels[cnum].sfxinfo && channels[cnum].origin == origin)
         {
@@ -369,7 +489,7 @@ void S_UnlinkSound(mobj_t *origin)
 
     if (origin)
     {
-        for (cnum = 0; cnum < numChannels; cnum++)
+        for (cnum = 0; cnum < snd_channels; cnum++)
         {
             if (channels[cnum].sfxinfo && channels[cnum].origin == origin)
             {
@@ -435,7 +555,7 @@ void S_UpdateSounds(const mobj_t *listener)
 
     I_DeferSoundUpdates();
 
-    for (cnum = 0; cnum < numChannels; ++cnum)
+    for (cnum = 0; cnum < snd_channels; ++cnum)
     {
         channel_t *c = &channels[cnum];
         sfxinfo_t *sfx = c->sfxinfo;
@@ -454,7 +574,6 @@ void S_UpdateSounds(const mobj_t *listener)
             {
                 // initialize parameters
                 int volume = snd_SfxVolume;
-                int pitch = c->pitch; // haleyjd 06/03/06: use channel's pitch!
                 int sep = NORM_SEP;
                 int pri = c->o_priority; // haleyjd 09/27/06: priority
 
@@ -464,7 +583,7 @@ void S_UpdateSounds(const mobj_t *listener)
                 if (c->origin && listener != c->origin) // killough 3/20/98
                 {
                     if (!S_AdjustSoundParams(listener, c->origin, c->volume,
-                                             &volume, &sep, &pitch, &pri))
+                                             &volume, &sep, &pri))
                     {
                         S_StopChannel(cnum);
                     }
@@ -474,6 +593,8 @@ void S_UpdateSounds(const mobj_t *listener)
                         c->priority = pri; // haleyjd
                     }
                 }
+
+                I_UpdateRumbleParams(listener, c->origin, c->handle);
             }
             else // if channel is allocated but sound has stopped, free it
             {
@@ -484,16 +605,7 @@ void S_UpdateSounds(const mobj_t *listener)
 
     I_UpdateListenerParams(listener);
     I_ProcessSoundUpdates();
-}
-
-void S_UpdateMusic(void)
-{
-    if (nomusicparm)
-    {
-        return;
-    }
-
-    I_UpdateMusic();
+    I_UpdateRumble();
 }
 
 void S_SetMusicVolume(int volume)
@@ -533,6 +645,8 @@ void S_SetSfxVolume(int volume)
     snd_SfxVolume = volume;
 }
 
+static extra_music_t extra_music;
+
 static int current_musicnum = -1;
 
 void S_ChangeMusic(int musicnum, int looping)
@@ -573,8 +687,21 @@ void S_ChangeMusic(int musicnum, int looping)
         music->lumpnum = W_GetNumForName(namebuf);
     }
 
+    int old_lumpnum = music->lumpnum;
+
     // load & register it
     music->data = W_CacheLumpNum(music->lumpnum, PU_STATIC);
+    if (extra_music && trakinfo_found)
+    {
+        const char *extra =
+            S_GetExtra(music->data, W_LumpLength(music->lumpnum), extra_music);
+        if (extra)
+        {
+            music->lumpnum = W_GetNumForName(extra);
+            Z_Free(music->data);
+            music->data = W_CacheLumpNum(music->lumpnum, PU_STATIC);
+        }
+    }
     // julian: added lump length
     music->handle = I_RegisterSong(music->data, W_LumpLength(music->lumpnum));
 
@@ -582,8 +709,12 @@ void S_ChangeMusic(int musicnum, int looping)
     I_PlaySong((void *)music->handle, looping);
 
     // [crispy] log played music
-    I_Printf(VB_INFO, "S_ChangeMusic: %.8s (%s)", lumpinfo[music->lumpnum].name,
-             W_WadNameForLump(music->lumpnum));
+    I_Printf(VB_DEBUG, "S_ChangeMusic: %.8s (%s), %s",
+             lumpinfo[music->lumpnum].name,
+             W_WadNameForLump(music->lumpnum),
+             I_MusicFormat());
+
+    music->lumpnum = old_lumpnum;
 
     mus_playing = music;
 
@@ -624,13 +755,28 @@ void S_ChangeMusInfoMusic(int lumpnum, int looping)
     music->lumpnum = lumpnum;
 
     music->data = W_CacheLumpNum(music->lumpnum, PU_STATIC);
+    if (extra_music && trakinfo_found)
+    {
+        const char *extra =
+            S_GetExtra(music->data, W_LumpLength(music->lumpnum), extra_music);
+        if (extra)
+        {
+            music->lumpnum = W_GetNumForName(extra);
+            Z_Free(music->data);
+            music->data = W_CacheLumpNum(music->lumpnum, PU_STATIC);
+        }
+    }
     music->handle = I_RegisterSong(music->data, W_LumpLength(music->lumpnum));
 
     I_PlaySong((void *)music->handle, looping);
 
     // [crispy] log played music
-    I_Printf(VB_INFO, "S_ChangeMusInfoMusic: %.8s (%s)",
-             lumpinfo[music->lumpnum].name, W_WadNameForLump(music->lumpnum));
+    I_Printf(VB_DEBUG, "S_ChangeMusInfoMusic: %.8s (%s), %s",
+             lumpinfo[music->lumpnum].name,
+             W_WadNameForLump(music->lumpnum),
+             I_MusicFormat());
+
+    music->lumpnum = lumpnum;
 
     mus_playing = music;
 
@@ -708,7 +854,7 @@ void S_Start(void)
     // jff 1/22/98 skip sound init if sound not enabled
     if (!nosfxparm)
     {
-        for (cnum = 0; cnum < numChannels; ++cnum)
+        for (cnum = 0; cnum < snd_channels; ++cnum)
         {
             if (channels[cnum].sfxinfo)
             {
@@ -811,7 +957,6 @@ void S_Init(int sfxVolume, int musicVolume)
         S_SetSfxVolume(sfxVolume);
 
         // Reset channel memory
-        numChannels = default_numChannels;
         memset(channels, 0, sizeof(channels));
         memset(sobjs, 0, sizeof(sobjs));
     }
@@ -827,6 +972,12 @@ void S_Init(int sfxVolume, int musicVolume)
     }
 }
 
+void S_BindSoundVariables(void)
+{
+    BIND_NUM(extra_music, EXMUS_OFF, EXMUS_OFF, EXMUS_ORIGINAL,
+             "Extra soundtrack (0 = Off; 1 = Remix; 2 = Original");
+}
+
 //----------------------------------------------------------------------------
 //
 // $Log: s_sound.c,v $
