@@ -22,28 +22,33 @@
 #ifndef __STSTUFF_H__
 #define __STSTUFF_H__
 
+#include "doomdef.h"
 #include "doomtype.h"
-#include "d_event.h"
+
+struct event_s;
+struct patch_s;
 
 // Size of statusbar.
 // Now sensitive for scaling.
 
 #define ST_HEIGHT 32
-#define ST_WIDTH  ORIGWIDTH
-#define ST_Y      (ORIGHEIGHT - ST_HEIGHT)
+#define ST_WIDTH  SCREENWIDTH
+#define ST_Y      (SCREENHEIGHT - ST_HEIGHT)
 
 //
 // STATUS BAR
 //
 
 // Called by main loop.
-boolean ST_Responder(event_t* ev);
+boolean ST_Responder(struct event_s *ev);
 
 // Called by main loop.
 void ST_Ticker(void);
 
 // Called by main loop.
-void ST_Drawer(boolean fullscreen, boolean refresh);
+void ST_Drawer(void);
+
+void ST_Erase(void);
 
 // Called when the console player is spawned on each level.
 void ST_Start(void);
@@ -51,32 +56,25 @@ void ST_Start(void);
 // Called by startup code.
 void ST_Init(void);
 
-// [crispy] forcefully initialize the status bar backing screen
-extern void ST_refreshBackground(boolean force);
+void ST_ResetPalette(void);
 
-// killough 5/2/98: moved from m_misc.c:
+extern boolean st_refresh_background;
 
-// [Alaux]
-extern int smooth_counts;
-extern int st_health;
-extern int st_armor;
+void ST_InitRes(void);
 
 extern int health_red;    // health amount less than which status is red
 extern int health_yellow; // health amount less than which status is yellow
 extern int health_green;  // health amount above is blue, below is green
-extern int armor_red;     // armor amount less than which status is red
-extern int armor_yellow;  // armor amount less than which status is yellow
-extern int armor_green;   // armor amount above is blue, below is green
-extern int ammo_red;      // ammo percent less than which status is red
-extern int ammo_yellow;   // ammo percent less is yellow more green
-extern int sts_always_red;// status numbers do not change colors
-extern int sts_pct_always_gray;// status percents do not change colors
-extern int sts_traditional_keys;  // display keys the traditional way
 
-extern int hud_backpack_thresholds; // backpack changes thresholds
-extern int hud_armor_type; // color of armor depends on type
+extern boolean palette_changes;
 
-extern int st_solidbackground;
+extern struct hudfont_s *stcfnt;
+extern struct patch_s **hu_font;
+
+void WI_UpdateWidgets(void);
+void WI_DrawWidgets(void);
+
+void ST_BindSTSVariables(void);
 
 #endif
 
