@@ -18,13 +18,14 @@
 //
 //-----------------------------------------------------------------------------
 
-#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "doomdef.h"
 #include "doomstat.h"
+#include "doomtype.h"
 #include "i_system.h"
+#include "m_array.h"
 #include "m_misc.h"
 #include "u_mapinfo.h"
 #include "u_scanner.h"
@@ -41,6 +42,8 @@ umapinfo_t U_mapinfo;
 
 umapinfo_t default_mapinfo;
 
+static level_t *secretlevels;
+
 static const char *const ActorNames[] =
 {
     "DoomPlayer", "ZombieMan", "ShotgunGuy", "Archvile", "ArchvileFire",
@@ -232,7 +235,7 @@ static void UpdateMapEntry(mapentry_t *mape, mapentry_t *newe)
     }
     if (newe->author)
     {
-        strcpy(mape->author, newe->author);
+        ReplaceString(&mape->author, newe->author);
     }
     if (newe->intertext)
     {
@@ -274,6 +277,14 @@ static void UpdateMapEntry(mapentry_t *mape, mapentry_t *newe)
     {
         strcpy(mape->enterpic, newe->enterpic);
     }
+    if (newe->exitanim[0])
+    {
+        strcpy(mape->exitanim, newe->exitanim);
+    }
+    if (newe->enteranim[0])
+    {
+        strcpy(mape->enteranim, newe->enteranim);
+    }
     if (newe->interbackdrop[0])
     {
         strcpy(mape->interbackdrop, newe->interbackdrop);
@@ -374,7 +385,7 @@ static int ParseLumpName(u_scanner_t *s, char *buffer)
     }
     strncpy(buffer, s->string, 8);
     buffer[8] = 0;
-    M_ForceUppercase(buffer);
+    M_StringToUpper(buffer);
     return 1;
 }
 
@@ -461,12 +472,13 @@ static int ParseStandardProperty(u_scanner_t *s, mapentry_t *mape)
                     if (U_MustGetToken(s, TK_StringConst))
                     {
                         key = strdup(s->string);
-                        key[0] = tolower(key[0]);
+                        key[0] = M_ToLower(key[0]);
                     }
                 }
             }
 
             M_AddEpisode(mape->mapname, lumpname, alttext, key);
+            mape->flags |= MapInfo_Episode;
 
             if (alttext)
             {
@@ -490,11 +502,13 @@ static int ParseStandardProperty(u_scanner_t *s, mapentry_t *mape)
     else if (!strcasecmp(pname, "nextsecret"))
     {
         status = ParseLumpName(s, mape->nextsecret);
-        if (!G_ValidateMapName(mape->nextsecret, NULL, NULL))
+        level_t level = {0};
+        if (!G_ValidateMapName(mape->nextsecret, &level.episode, &level.map))
         {
             U_Error(s, "Invalid map name %s", mape->nextsecret);
             status = 0;
         }
+        array_push(secretlevels, level);
     }
     else if (!strcasecmp(pname, "levelpic"))
     {
@@ -518,6 +532,7 @@ static int ParseStandardProperty(u_scanner_t *s, mapentry_t *mape)
         if (s->sc_boolean)
         {
             strcpy(mape->endpic, "$CAST");
+            mape->flags |= MapInfo_Endgame;
         }
         else
         {
@@ -530,6 +545,7 @@ static int ParseStandardProperty(u_scanner_t *s, mapentry_t *mape)
         if (s->sc_boolean)
         {
             strcpy(mape->endpic, "$BUNNY");
+            mape->flags |= MapInfo_Endgame;
         }
         else
         {
@@ -542,6 +558,7 @@ static int ParseStandardProperty(u_scanner_t *s, mapentry_t *mape)
         if (s->sc_boolean)
         {
             strcpy(mape->endpic, "!");
+            mape->flags |= MapInfo_Endgame;
         }
         else
         {
@@ -556,6 +573,14 @@ static int ParseStandardProperty(u_scanner_t *s, mapentry_t *mape)
     {
         status = ParseLumpName(s, mape->enterpic);
     }
+    else if (!strcasecmp(pname, "exitanim"))
+    {
+        status = ParseLumpName(s, mape->exitanim);
+    }
+    else if (!strcasecmp(pname, "enteranim"))
+    {
+        status = ParseLumpName(s, mape->enteranim);
+    }
     else if (!strcasecmp(pname, "nointermission"))
     {
         if (U_MustGetToken(s, TK_BoolConst))
@@ -810,7 +835,7 @@ void U_ParseMapInfo(int lumpnum)
 
                 G_ValidateMapName(parsed.mapname, &ep, &map);
 
-                strcpy(parsed.nextmap, MAPNAME(ep, map + 1));
+                strcpy(parsed.nextmap, MapName(ep, map + 1));
             }
         }
 
@@ -848,3 +873,16 @@ boolean U_CheckField(char *str)
 {
     return str && str[0] && strcmp(str, "-");
 }
+
+boolean U_IsSecretMap(int episode, int map)
+{
+    level_t *level;
+    array_foreach(level, secretlevels)
+    {
+        if (level->episode == episode && level->map == map)
+        {
+            return true;
+        }
+    }
+    return false;
+}
