@@ -30,6 +30,12 @@ typedef struct
     int tag;
 } bossaction_t;
 
+typedef enum
+{
+    MapInfo_Episode    = 0x0001,
+    MapInfo_Endgame    = 0x0002,
+} mapinfo_flags_t;
+
 typedef struct mapentry_s
 {
     char *mapname;
@@ -46,9 +52,12 @@ typedef struct mapentry_s
     char endpic[9];
     char exitpic[9];
     char enterpic[9];
+    char exitanim[9];
+    char enteranim[9];
     char interbackdrop[9];
     char intermusic[9];
     int partime;
+    mapinfo_flags_t flags;
     boolean nointermission;
     int numbossactions;
     bossaction_t *bossactions;
@@ -64,6 +73,7 @@ extern umapinfo_t U_mapinfo;
 extern umapinfo_t default_mapinfo;
 
 extern boolean EpiCustom;
+
 mapentry_t *G_LookupMapinfo(int episode, int map);
 
 boolean U_CheckField(char *str);
@@ -72,4 +82,6 @@ void U_ParseMapDefInfo(int lumpnum);
 
 void U_ParseMapInfo(int lumpnum);
 
+boolean U_IsSecretMap(int episode, int map);
+
 #endif
