@@ -21,12 +21,12 @@
 #include <string.h>
 
 #include "d_event.h"
+#include "d_deh.h"
 #include "d_player.h"
 #include "doomdef.h"
 #include "doomstat.h"
 #include "doomtype.h"
 #include "g_game.h"
-#include "hu_lib.h"
 #include "i_printf.h"
 #include "m_misc.h"
 #include "m_random.h"
@@ -34,16 +34,18 @@
 #include "mn_menu.h"
 #include "r_defs.h"
 #include "s_sound.h"
+#include "st_sbardef.h"
+#include "st_stuff.h"
 #include "sounds.h"
 #include "u_mapinfo.h"
+#include "v_fmt.h"
 #include "v_video.h"
 #include "w_wad.h"
+#include "wi_interlvl.h"
 #include "wi_stuff.h"
 #include "z_zone.h"
 
-// Ty 03/17/98: flag that new par times have been loaded in d_deh
-extern boolean deh_pars;  
-extern boolean um_pars;
+#define LARGENUMBER 1994
 
 //
 // Data needed to add patches to full screen intermission pics.
@@ -385,10 +387,306 @@ static int    num_lnames;
 
 static const char *exitpic, *enterpic;
 
+#define M_ARRAY_MALLOC(size) Z_Malloc((size), PU_LEVEL, NULL)
+#define M_ARRAY_REALLOC(ptr, size) Z_Realloc((ptr), (size), PU_LEVEL, NULL)
+#define M_ARRAY_FREE(ptr) Z_Free((ptr))
+#include "m_array.h"
+
+typedef struct
+{
+    interlevelframe_t *frames;
+    int x_pos;
+    int y_pos;
+    int frame_index;
+    boolean frame_start;
+    int duration_left;
+} wi_animationstate_t;
+
+typedef struct
+{
+    interlevel_t *interlevel_exiting;
+    interlevel_t *interlevel_entering;
+
+    wi_animationstate_t *states;
+    char *background_lump;
+} wi_animation_t;
+
+static wi_animation_t *animation;
+
 //
 // CODE
 //
 
+static boolean CheckConditions(interlevelcond_t *conditions,
+                               boolean enteringcondition)
+{
+    boolean conditionsmet = true;
+
+    int map, episode;
+    if (enteringcondition)
+    {
+        map = wbs->next + 1;
+        episode = wbs->nextep + 1;
+    }
+    else
+    {
+        map = wbs->last + 1;
+        episode = wbs->epsd + 1;
+    }
+
+    interlevelcond_t *cond;
+    array_foreach(cond, conditions)
+    {
+        switch (cond->condition)
+        {
+            case AnimCondition_MapNumGreater:
+                conditionsmet &= (map > cond->param);
+                break;
+
+            case AnimCondition_MapNumEqual:
+                conditionsmet &= (map == cond->param);
+                break;
+
+            case AnimCondition_MapVisited:
+                {
+                    boolean result = false;
+                    level_t *level;
+                    array_foreach(level, wbs->visitedlevels)
+                    {
+                        if (level->map == cond->param)
+                        {
+                            result = true;
+                            break;
+                        }
+                    }
+                    conditionsmet &= result;
+                }
+                break;
+
+            case AnimCondition_MapNotSecret:
+                conditionsmet &= !U_IsSecretMap(episode, map);
+                break;
+
+            case AnimCondition_SecretVisited:
+                conditionsmet &= wbs->didsecret;
+                break;
+
+            case AnimCondition_Tally:
+                conditionsmet &= !enteringcondition;
+                break;
+
+            case AnimCondition_IsEntering:
+                conditionsmet &= enteringcondition;
+                break;
+
+            default:
+                break;
+        }
+    }
+    return conditionsmet;
+}
+
+static boolean UpdateAnimation(void)
+{
+    if (!animation || !animation->states)
+    {
+        return false;
+    }
+
+    wi_animationstate_t *state;
+    array_foreach(state, animation->states)
+    {
+        interlevelframe_t *frame = &state->frames[state->frame_index];
+
+        if (frame->type & Frame_Infinite)
+        {
+            continue;
+        }
+
+        if (state->duration_left == 0)
+        {
+            if (!state->frame_start)
+            {
+                state->frame_index++;
+                if (state->frame_index == array_size(state->frames))
+                {
+                    state->frame_index = 0;
+                }
+
+                frame = &state->frames[state->frame_index];
+            }
+
+            int tics = 1;
+            switch (frame->type)
+            {
+                case Frame_RandomStart:
+                    if (state->frame_start)
+                    {
+                        tics = M_Random() % frame->duration;
+                        break;
+                    }
+                    // fall through
+                case Frame_FixedDuration:
+                    tics = frame->duration;
+                    break;
+
+                case Frame_RandomDuration:
+                    tics = M_Random() % frame->maxduration;
+                    tics = BETWEEN(frame->duration, frame->maxduration, tics);
+                    break;
+
+                default:
+                    break;
+            }
+
+            state->duration_left = MAX(tics, 1);
+        }
+
+        state->duration_left--;
+        state->frame_start = false;
+    }
+
+    return true;
+}
+
+static boolean DrawAnimation(void)
+{
+    if (!animation)
+    {
+        return false;
+    }
+
+    if (animation->background_lump)
+    {
+        V_DrawPatchFullScreen(
+            V_CachePatchName(animation->background_lump, PU_CACHE));
+    }
+
+    wi_animationstate_t *state;
+    array_foreach(state, animation->states)
+    {
+        interlevelframe_t *frame = &state->frames[state->frame_index];
+        int lumpnum = W_CheckNumForName(frame->image_lump);
+        if (lumpnum < 0)
+        {
+            lumpnum = (W_CheckNumForName)(frame->image_lump, ns_sprites);
+        }
+        V_DrawPatch(state->x_pos, state->y_pos,
+                    V_CachePatchNum(lumpnum, PU_CACHE));
+    }
+
+    return true;
+}
+
+static wi_animationstate_t *SetupAnimationStates(interlevellayer_t *layers,
+                                                 boolean enteringcondition)
+{
+    wi_animationstate_t *states = NULL;
+
+    interlevellayer_t *layer;
+    array_foreach(layer, layers)
+    {
+        if (!CheckConditions(layer->conditions, enteringcondition))
+        {
+            continue;
+        }
+
+        interlevelanim_t *anim;
+        array_foreach(anim, layer->anims)
+        {
+            if (!CheckConditions(anim->conditions, enteringcondition))
+            {
+                continue;
+            }
+
+            wi_animationstate_t state = {0};
+            state.x_pos = anim->x_pos;
+            state.y_pos = anim->y_pos;
+            state.frames = anim->frames;
+            state.frame_start = true;
+            array_push(states, state);
+        }
+    }
+
+    return states;
+}
+
+static boolean SetupAnimation(boolean enteringcondition)
+{
+    if (!animation)
+    {
+        return false;
+    }
+
+    interlevel_t *interlevel = NULL;
+    if (!enteringcondition)
+    {
+        if (animation->interlevel_exiting)
+        {
+            interlevel = animation->interlevel_exiting;
+        }
+    }
+    else if (animation->interlevel_entering)
+    {
+        interlevel = animation->interlevel_entering;
+    }
+
+    if (interlevel)
+    {
+        animation->states =
+            SetupAnimationStates(interlevel->layers, enteringcondition);
+        animation->background_lump = interlevel->background_lump;
+        return true;
+    }
+
+    animation->states = NULL;
+    animation->background_lump = NULL;
+    return false;
+}
+
+static boolean NextLocAnimation(void)
+{
+    if (animation && animation->interlevel_entering
+        && !(demorecording || demoplayback))
+    {
+        return true;
+    }
+
+    return false;
+}
+
+static boolean SetupMusic(boolean enteringcondition)
+{
+    if (!animation)
+    {
+        return false;
+    }
+
+    interlevel_t *interlevel = NULL;
+    if (!enteringcondition)
+    {
+        if (animation->interlevel_exiting)
+        {
+            interlevel = animation->interlevel_exiting;
+        }
+    }
+    else if (animation->interlevel_entering)
+    {
+        interlevel = animation->interlevel_entering;
+    }
+
+    if (interlevel)
+    {
+        int musicnum = W_GetNumForName(interlevel->music_lump);
+        if (musicnum >= 0)
+        {
+            S_ChangeMusInfoMusic(musicnum, true);
+            return true;
+        }
+    }
+
+    return false;
+}
 
 // ====================================================================
 // WI_slamBackground
@@ -410,7 +708,7 @@ void WI_slamBackground(void)
   else
     M_snprintf(name, sizeof(name), "WIMAP%d", wbs->epsd);
 
-  V_DrawPatchFullScreen(W_CacheLumpName(name, PU_CACHE));
+  V_DrawPatchFullScreen(V_CachePatchName(name, PU_CACHE));
 }
 
 // ====================================================================
@@ -458,7 +756,7 @@ static void WI_drawLF(void)
   }
   else if (wbs->lastmapinfo && wbs->lastmapinfo->levelpic[0])
   {
-    patch_t* lpic = W_CacheLumpName(wbs->lastmapinfo->levelpic, PU_CACHE);
+    patch_t* lpic = V_CachePatchName(wbs->lastmapinfo->levelpic, PU_CACHE);
 
     V_DrawPatch((SCREENWIDTH - SHORT(lpic->width))/2,
                y, lpic);
@@ -512,12 +810,12 @@ static void WI_drawEL(void)
   }
   else if (wbs->nextmapinfo && wbs->nextmapinfo->levelpic[0])
   {
-    patch_t* lpic = W_CacheLumpName(wbs->nextmapinfo->levelpic, PU_CACHE);
+    patch_t* lpic = V_CachePatchName(wbs->nextmapinfo->levelpic, PU_CACHE);
 
-    y += (5 * SHORT(lpic->height)) / 4;
+    if (SHORT(lpic->height) < SCREENHEIGHT)
+      y += (5 * SHORT(lpic->height)) / 4;
 
-    V_DrawPatch((SCREENWIDTH - SHORT(lpic->width))/2,
-               y, lpic);
+    V_DrawPatch((SCREENWIDTH - SHORT(lpic->width))/2, y, lpic);
   }
   // [FG] prevent crashes for levels without name graphics
   else if (wbs->next >= 0 && wbs->next < num_lnames && lnames[wbs->next] != NULL)
@@ -588,14 +886,19 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
 // ====================================================================
 // WI_initAnimatedBack
 // Purpose: Initialize pointers and styles for background animation
-// Args:    none
+// Args:    firstcall -- [Woof!] check if animations needs to be reset
 // Returns: void
 //
-static void WI_initAnimatedBack(void)
+static void WI_initAnimatedBack(boolean firstcall)
 {
   int   i;
   anim_t* a;
 
+  if (SetupAnimation(state != StatCount))
+  {
+      return;
+  }
+
   if (exitpic)
     return;
   if (enterpic && entering)
@@ -612,7 +915,11 @@ static void WI_initAnimatedBack(void)
       a = &anims[wbs->epsd][i];
 
       // init variables
-      a->ctr = -1;
+      // [Woof!] Do not reset animation timers upon switching to "Entering" state
+      // via WI_initShowNextLoc. Fixes notable blinking of Tower of Babel drawing
+      // and the rest of animations from being restarted.
+      if (firstcall)
+        a->ctr = -1;
   
       // specify the next time to draw it
       if (a->type == ANIM_ALWAYS)
@@ -638,6 +945,11 @@ static void WI_updateAnimatedBack(void)
   int   i;
   anim_t* a;
 
+  if (UpdateAnimation())
+  {
+      return;
+  }
+
   if (exitpic)
     return;
   if (enterpic && state != StatCount)
@@ -700,6 +1012,11 @@ static void WI_drawAnimatedBack(void)
   int     i;
   anim_t*   a;
 
+  if (DrawAnimation())
+  {
+      return;
+  }
+
   if (exitpic)
     return;
   if (enterpic && state != StatCount)
@@ -767,7 +1084,7 @@ WI_drawNum
     }
 
   // if non-number, do not draw it
-  if (n == 1994)
+  if (n == LARGENUMBER)
     return 0;
 
   // draw the new number
@@ -851,7 +1168,7 @@ WI_drawTime
       // [FG] print at most in hhhh:mm:ss format
       if ((n = (t / div)))
       {
-        x = WI_drawNum(x, y, n, -1);
+        WI_drawNum(x, y, n, -1);
       }
     }
   else
@@ -989,6 +1306,8 @@ static boolean    snl_pointeron = false;
 //
 static void WI_initShowNextLoc(void)
 {
+  SetupMusic(true);
+
   if (gamemapinfo)
   {
     if (gamemapinfo->endpic[0])
@@ -1013,7 +1332,7 @@ static void WI_initShowNextLoc(void)
   acceleratestage = 0;
   cnt = SHOWNEXTLOCDELAY * TICRATE;
 
-  WI_initAnimatedBack();
+  WI_initAnimatedBack(false);
 }
 
 
@@ -1069,7 +1388,9 @@ static void WI_drawShowNextLoc(void)
           WI_drawEL();  // "Entering..." if not E1 or E2
           return;
         }
-  
+
+      if (!animation || !animation->states)
+      {
       last = (wbs->last == 8) ? wbs->next - 1 : wbs->last;
 
       // draw a splat on taken cities.
@@ -1083,6 +1404,7 @@ static void WI_drawShowNextLoc(void)
       // draw flashing ptr
       if (snl_pointeron)
         WI_drawOnLnode(wbs->next, yah); 
+      }
     }
 
   // draws which level you are entering..
@@ -1167,7 +1489,7 @@ static void WI_initDeathmatchStats(void)
           dm_totals[i] = 0;
         }
     }
-  WI_initAnimatedBack();
+  WI_initAnimatedBack(true);
 }
 
 
@@ -1263,6 +1585,8 @@ static void WI_updateDeathmatchStats(void)
           {   
             S_StartSound(0, sfx_slop);
 
+            if (NextLocAnimation())
+              WI_initShowNextLoc();
             if ( gamemode == commercial)
               WI_initNoState();
             else
@@ -1408,7 +1732,7 @@ static void WI_initNetgameStats(void)
 
   dofrags = !!dofrags; // set to true or false - did we have frags?
 
-  WI_initAnimatedBack();
+  WI_initAnimatedBack(true);
 }
 
 
@@ -1566,6 +1890,9 @@ static void WI_updateNetgameStats(void)
               if (acceleratestage)
                 {
                   S_StartSound(0, sfx_sgcock);
+
+                  if (NextLocAnimation())
+                    WI_initShowNextLoc();
                   if ( gamemode == commercial )
                     WI_initNoState();
                   else
@@ -1663,7 +1990,7 @@ static void WI_initStats(void)
   cnt_time = cnt_par = cnt_total_time = -1;
   cnt_pause = TICRATE;
 
-  WI_initAnimatedBack();
+  WI_initAnimatedBack(true);
 }
 
 // ====================================================================
@@ -1733,7 +2060,7 @@ static void WI_updateStats(void)
           // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
           // [FG] Intermission screen secrets desync
           // http://prboom.sourceforge.net/mbf-bugs.html
-          if ((!wbs->maxsecret && demo_version < 203) ||
+          if ((!wbs->maxsecret && demo_version < DV_MBF) ||
               cnt_secret[0] >= (wbs->maxsecret ? 
                                 (plrs[me].ssecret * 100) / wbs->maxsecret : 100))
             {
@@ -1767,10 +2094,10 @@ static void WI_updateStats(void)
 
                 // This check affects demo compatibility with PrBoom+
                 if ((cnt_time >= plrs[me].stime / TICRATE) &&
-                    (demo_version < 203 || cnt_total_time >= wbs->totaltimes / TICRATE)
+                    (demo_version < DV_MBF || cnt_total_time >= wbs->totaltimes / TICRATE)
                    )
                   {
-                    if (demo_version < 203)
+                    if (demo_version < DV_MBF)
                       cnt_total_time = wbs->totaltimes / TICRATE;
                     S_StartSound(0, sfx_barexp);
                     sp_state++;
@@ -1784,7 +2111,9 @@ static void WI_updateStats(void)
                 {
                   S_StartSound(0, sfx_sgcock);
 
-                  if (gamemode == commercial)
+                  if (NextLocAnimation())
+                    WI_initShowNextLoc();
+                  else if (gamemode == commercial)
                     WI_initNoState();
                   else
                     WI_initShowNextLoc();
@@ -1811,7 +2140,7 @@ static void WI_drawStats(void)
 {
   // line height
   int lh; 
-  int maplump = W_CheckNumForName(MAPNAME(wbs->epsd + 1, wbs->last + 1));
+  int maplump = W_CheckNumForName(MapName(wbs->epsd + 1, wbs->last + 1));
 
   lh = (3*SHORT(num[0]->height))/2;
 
@@ -1907,7 +2236,7 @@ void WI_Ticker(void)
   // counter for general background animation
   bcnt++;  
 
-  if (bcnt == 1)
+  if (bcnt == 1 && !SetupMusic(false))
     {
       // intermission music
       if ( gamemode == commercial )
@@ -1925,6 +2254,7 @@ void WI_Ticker(void)
       else
         if (netgame) WI_updateNetgameStats();
         else WI_updateStats();
+      WI_UpdateWidgets();
       break;
   
     case ShowNextLoc:
@@ -1973,7 +2303,7 @@ void WI_loadData(void)
           M_snprintf(name, sizeof(name), "CWILV%2.2d", i);
           if (W_CheckNumForName(name) != -1)
           {
-          lnames[i] = W_CacheLumpName(name, PU_STATIC);
+          lnames[i] = V_CachePatchName(name, PU_STATIC);
           }
           else
           {
@@ -1990,7 +2320,7 @@ void WI_loadData(void)
           M_snprintf(name, sizeof(name), "WILV%d%d", wbs->epsd, i);
           if (W_CheckNumForName(name) != -1)
           {
-          lnames[i] = W_CacheLumpName(name, PU_STATIC);
+          lnames[i] = V_CachePatchName(name, PU_STATIC);
           }
           else
           {
@@ -1999,13 +2329,13 @@ void WI_loadData(void)
         }
 
       // you are here
-      yah[0] = W_CacheLumpName("WIURH0", PU_STATIC);
+      yah[0] = V_CachePatchName("WIURH0", PU_STATIC);
 
       // you are here (alt.)
-      yah[1] = W_CacheLumpName("WIURH1", PU_STATIC);
+      yah[1] = V_CachePatchName("WIURH1", PU_STATIC);
 
       // splat
-      splat[0] = W_CacheLumpName("WISPLAT", PU_STATIC); 
+      splat[0] = V_CachePatchName("WISPLAT", PU_STATIC); 
   
       if (wbs->epsd < 3)
         {
@@ -2019,7 +2349,7 @@ void WI_loadData(void)
                     {
                       // animations
                       M_snprintf(name, sizeof(name), "WIA%d%.2d%.2d", wbs->epsd, j, i);
-                      a->p[i] = W_CacheLumpName(name, PU_STATIC);
+                      a->p[i] = V_CachePatchName(name, PU_STATIC);
                     }
                   else
                     {
@@ -2034,32 +2364,32 @@ void WI_loadData(void)
   // More hacks on minus sign.
   // [FG] allow playing with the Doom v1.2 IWAD which is missing the WIMINUS lump
   if (W_CheckNumForName("WIMINUS") >= 0)
-  wiminus = W_CacheLumpName("WIMINUS", PU_STATIC); 
+  wiminus = V_CachePatchName("WIMINUS", PU_STATIC); 
 
   for (i=0;i<10;i++)
     {
       // numbers 0-9
       M_snprintf(name, sizeof(name), "WINUM%d", i);
-      num[i] = W_CacheLumpName(name, PU_STATIC);
+      num[i] = V_CachePatchName(name, PU_STATIC);
     }
 
   // percent sign
-  percent = W_CacheLumpName("WIPCNT", PU_STATIC);
+  percent = V_CachePatchName("WIPCNT", PU_STATIC);
 
   // "finished"
-  finished = W_CacheLumpName("WIF", PU_STATIC);
+  finished = V_CachePatchName("WIF", PU_STATIC);
 
   // "entering"
-  entering = W_CacheLumpName("WIENTER", PU_STATIC);
+  entering = V_CachePatchName("WIENTER", PU_STATIC);
 
   // "kills"
-  kills = W_CacheLumpName("WIOSTK", PU_STATIC);   
+  kills = V_CachePatchName("WIOSTK", PU_STATIC);   
 
   // "scrt"
-  secret = W_CacheLumpName("WIOSTS", PU_STATIC);
+  secret = V_CachePatchName("WIOSTS", PU_STATIC);
 
   // "secret"
-  sp_secret = W_CacheLumpName("WISCRT2", PU_STATIC);
+  sp_secret = V_CachePatchName("WISCRT2", PU_STATIC);
 
   // Yuck. // Ty 03/27/98 - got that right :)  
   // french is an enum=1 always true.
@@ -2072,47 +2402,47 @@ void WI_loadData(void)
   //      items = W_CacheLumpName("WIOSTI", PU_STATIC);
   //    } else
 
-  items = W_CacheLumpName("WIOSTI", PU_STATIC);
+  items = V_CachePatchName("WIOSTI", PU_STATIC);
 
   // "frgs"
-  frags = W_CacheLumpName("WIFRGS", PU_STATIC);    
+  frags = V_CachePatchName("WIFRGS", PU_STATIC);    
 
   // ":"
-  colon = W_CacheLumpName("WICOLON", PU_STATIC); 
+  colon = V_CachePatchName("WICOLON", PU_STATIC); 
 
   // "time"
-  witime = W_CacheLumpName("WITIME", PU_STATIC);
+  witime = V_CachePatchName("WITIME", PU_STATIC);
 
   // "sucks"
-  sucks = W_CacheLumpName("WISUCKS", PU_STATIC);  
+  sucks = V_CachePatchName("WISUCKS", PU_STATIC);  
 
   // "par"
-  par = W_CacheLumpName("WIPAR", PU_STATIC);   
+  par = V_CachePatchName("WIPAR", PU_STATIC);   
 
   // "killers" (vertical)
-  killers = W_CacheLumpName("WIKILRS", PU_STATIC);
+  killers = V_CachePatchName("WIKILRS", PU_STATIC);
   
   // "victims" (horiz)
-  victims = W_CacheLumpName("WIVCTMS", PU_STATIC);
+  victims = V_CachePatchName("WIVCTMS", PU_STATIC);
 
   // "total"
-  total = W_CacheLumpName("WIMSTT", PU_STATIC);   
+  total = V_CachePatchName("WIMSTT", PU_STATIC);   
 
   // your face
-  star = W_CacheLumpName("STFST01", PU_STATIC);
+  star = V_CachePatchName("STFST01", PU_STATIC);
 
   // dead face
-  bstar = W_CacheLumpName("STFDEAD0", PU_STATIC);    
+  bstar = V_CachePatchName("STFDEAD0", PU_STATIC);    
 
   for (i=0 ; i<MAXPLAYERS ; i++)
     {
       // "1,2,3,4"
       M_snprintf(name, sizeof(name), "STPB%d", i);
-      p[i] = W_CacheLumpName(name, PU_STATIC);
+      p[i] = V_CachePatchName(name, PU_STATIC);
 
       // "1,2,3,4"
       M_snprintf(name, sizeof(name), "WIBP%d", i + 1);
-      bp[i] = W_CacheLumpName(name, PU_STATIC);
+      bp[i] = V_CachePatchName(name, PU_STATIC);
     }
 }
 
@@ -2125,7 +2455,6 @@ void WI_loadData(void)
 //
 void WI_Drawer (void)
 {
-  extern void WI_DrawTimeWidget(void);
   switch (state)
     {
     case StatCount:
@@ -2137,7 +2466,7 @@ void WI_Drawer (void)
         else
           WI_drawStats();
       // [FG] draw Time widget on intermission screen
-      WI_DrawTimeWidget();
+      WI_DrawWidgets();
       break;
   
     case ShowNextLoc:
@@ -2215,8 +2544,43 @@ void WI_Start(wbstartstruct_t* wbstartstruct)
 {
   WI_initVariables(wbstartstruct);
 
-  exitpic = (wbs->lastmapinfo && wbs->lastmapinfo->exitpic[0]) ? wbs->lastmapinfo->exitpic : NULL;
-  enterpic = (wbs->nextmapinfo && wbs->nextmapinfo->enterpic[0]) ? wbs->nextmapinfo->enterpic : NULL;
+  exitpic = NULL;
+  enterpic = NULL;
+  animation = NULL;
+
+  if (wbs->lastmapinfo)
+  {
+      if (wbs->lastmapinfo->exitpic[0])
+      {
+          exitpic = wbs->lastmapinfo->exitpic;
+      }
+      if (wbs->lastmapinfo->exitanim[0])
+      {
+          if (!animation)
+          {
+              animation = Z_Calloc(1, sizeof(*animation), PU_LEVEL, NULL);
+          }
+          animation->interlevel_exiting =
+              WI_ParseInterlevel(wbs->lastmapinfo->exitanim);
+      }
+  }
+
+  if (wbs->nextmapinfo)
+  {
+      if (wbs->nextmapinfo->enterpic[0])
+      {
+          enterpic = wbs->nextmapinfo->enterpic;
+      }
+      if (wbs->nextmapinfo->enteranim[0])
+      {
+          if (!animation)
+          {
+              animation = Z_Calloc(1, sizeof(*animation), PU_LEVEL, NULL);
+          }
+          animation->interlevel_entering =
+              WI_ParseInterlevel(wbs->nextmapinfo->enteranim);
+      }
+  }
 
   WI_loadData();
 
